package facets.core.app;
import facets.core.superficial.SFacet;
import facets.core.superficial.app.SHost;
/**
{@link SHost} that can create and set application layouts. 
<p>Layouts are returned by 
{@link #newDesktopLayout(SFacet, LayoutFeatures)}
	 */
public interface AppHost extends SHost{
	/**
	Defines layout to be returned from
	 {@link AppHost#newDesktopLayout(SFacet,LayoutFeatures)}.
	 <p><code>null</code> implementations are allowed for all methods.  
	*/
	public interface LayoutFeatures{
		/**
		Will be displayed whenever this layout is active and should include
	 any desired application-level menus. 
		 */
		SFacet[]menus();
		/**
		To appear above content area. 
		 */
		SFacet toolbar();
		/**
		To appear beside content area. 
		 */
		SFacet sidebar();
		/**
		To appear below content area. 
		 */
		SFacet status();
		/**
		Will be returned via {@link #activeServices()} when
	the layout was the last set with {@link SHost#setLayout(facets.core.superficial.app.SHost.FacetLayout)}. 
		 */
		SurfaceServices services();
		/**
		To appear in sash pane with content area. 
		 */
		SFacet extras();
	}
	/**
	Create a layout containing the features passed. 
	 */
	FacetLayout newDesktopLayout(SFacet contentFacet,LayoutFeatures features);
	/**
	Return suitable services for facet in the active layout. 
	<p>These will generally have been passed to methods such as 
	{@link #newDesktopLayout(SFacet, LayoutFeatures)}, 
	enabling a hosted {@link AppSurface} to delegate definition 
	to an {@link facets.core.app.AppSurface.Contenter}. 
	 */
	SurfaceServices activeServices();
	/**
	Run the application which created the {@link AppHost}. 
	<p>Allows call to {@link AppSurface#openApp()} from a	GUI thread.
	 */
	void runHostedApp();
	/**
	Create a suitable {@link Dialogs} for use by an {@link AppSurface}. 
	@param app should host the dialogs launched
	 */
	abstract Dialogs newDialogs(ActionAppSurface app);
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

