package facets.core.app;
import static facets.core.app.AppConstants.*;
import facets.core.app.DialogSurface.Contenter;
import facets.core.app.DialogSurface.DialogActions;
import facets.core.superficial.STrigger;
import facets.core.superficial.STrigger.Coupler;
import facets.core.superficial.app.SContenter;
/**
Convenience extension of {@link DialogSurface.DialogActions}. 
<p>{@link DialogActionDefaults} extends its superclass by defining a range
of {@link STrigger}s representing commonly required top-level dialog 
actions; class methods return instances defining convenience combinations.  
 */
public abstract class DialogActionDefaults extends DialogActions{
	private final static int TRIGGER_OK=0,TRIGGER_CANCEL=1,TRIGGER_APPLY=2,
		TRIGGER_CLOSE=3,TRIGGERS=4;
	final public static DialogActions newOkCancel() {
		return new DialogActionDefaults(){
			public STrigger[]newTriggers(){
				return new STrigger[]{
						newTrigger(TRIGGER_OK),
						newTrigger(TRIGGER_CANCEL),
				};
			}
		};
	}
	final public static DialogActions newCancel() {
		return new DialogActionDefaults(){
			public STrigger[]newTriggers(){
				return new STrigger[]{
						newTrigger(TRIGGER_CANCEL),
				};
			}
		};
	}
	final public static DialogActions newOk() {
		return new DialogActionDefaults(){
			public STrigger[]newTriggers(){
				return new STrigger[]{
						newTrigger(TRIGGER_OK),
				};
			}
		};
	}
	final public static DialogActions newApplyOkCancel() {
		return new DialogActionDefaults(){
			public STrigger[]newTriggers(){
				return new STrigger[]{
						newTrigger(TRIGGER_APPLY),
						newTrigger(TRIGGER_OK),
						newTrigger(TRIGGER_CANCEL),
				};
			}
		};
	}
	/**
	Returns triggers for various actions. 
	<p>The method can be overridden to implement further action types. 
	@param type constant determining the action to be performed
	 */
	protected STrigger newTrigger(int type){
		final DialogSurface surface=surface();
		final SContenter[]contents=surface.contents();
		return type==TRIGGER_OK?new STrigger(TITLE_OK,new Coupler(){
				public void fired(STrigger t){
					for(int i=0;i<contents.length;i++)
						((Contenter)contents[i]).applyChanges();
					surface.dismissHostWindow(); 
				}
			})
			:type==TRIGGER_CANCEL?new STrigger(TITLE_CANCEL,new Coupler(){
				public void fired(STrigger t){
					for(int i=0;i<contents.length;i++)
						((Contenter)contents[i]).reverseChanges();
					surface.dismissHostWindow(); 
				}
			})
			:type==TRIGGER_APPLY?new STrigger(TITLE_APPLY,new Coupler(){
				public void fired(STrigger t){
					for(int i=0;i<contents.length;i++)
						((Contenter)contents[i]).applyChanges();
				}
			})
			:new STrigger(TITLE_CLOSE,new Coupler(){
				public void fired(STrigger t){
					surface.dismissHostWindow(); 
				}
			});
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
