package facets.core.app;
import static facets.core.app.ActionViewerTarget.Action.*;
import static facets.util.Objects.*;
import static facets.util.datatree.Nodes.*;
import static java.util.Arrays.*;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.STarget;
import facets.core.superficial.STextual;
import facets.core.superficial.app.SSelection;
import facets.core.superficial.app.SView;
import facets.core.superficial.app.SViewer;
import facets.core.superficial.app.ViewableAction;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.Objects;
import facets.util.OffsetPath;
import facets.util.Stateful;
import facets.util.Strings;
import facets.util.Util;
import facets.util.datatree.DataConstants;
import facets.util.datatree.DataNode;
import facets.util.datatree.XmlPolicy;
import facets.util.datatree.NodeList;
import facets.util.datatree.Nodes;
import facets.util.datatree.TypedNode;
import facets.util.datatree.ValueNode;
import facets.util.datatree.XmlDocRoot;
import facets.util.datatree.Nodes.TreeRoot;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
/**
{@link StatefulViewable} for tree-type content. 
<p>{@link NodeViewable} extends its superclass by 
<ul><li>requiring its {@link #framed} content to be a {@link TypedNode}
<li>providing useful implementations based on {@link TypedNode}
of all superclass methods, enabling its direct use by application code
</ul> 
 */
public class NodeViewable extends StatefulViewable<TypedNode>{
	private int selections;
	private TypedNode rootState;
	private boolean treeChanged,pastingInto;
  private static String newUseTitle(TypedNode node){
    String title=node.title();
    return title!=null&&!title.equals("")?title:"Untitled "+node.type();
  }
	public NodeViewable(TypedNode node){
		this(node,null);
	}
	public NodeViewable(TypedNode root,ClipperSource clipperSource){
	  super(newUseTitle(root),root,clipperSource);
	  if(clipperSource!=null)copyContentState();
	  setSelection(PathSelection.newMinimal(root));
	}
	protected final void copyContentState(){
		TypedNode src=(TypedNode)framed,
			copy=clipper==null?src:(TypedNode)src.copyState();
		treeChanged=!copy.stateEquals(rootState);
		rootState=copy;
	}
	/**
	Default implementation for {@link TypedNode} tree. 
	 */
	public SFrameTarget selectionFrame(){
	  final Object selection=selection().selections()[0];
	  return selection instanceof TypedNode?new NodeViewable((TypedNode)selection)
      :new SFrameTarget(title()+" #"+selections++,selection){
  			protected STarget[]lazyElements(){
  				STarget valueTextual=new STextual("Value",selection+"",
  						new STextual.Coupler());
					return new STarget[]{valueTextual};
  			}
  		};
	}
	protected STarget[]lazyElements(){
		TypedNode node=(TypedNode)framed;
		STarget valueTextual=new STextual("Node",node
				+" descendants="+(Nodes.descendants(node).length-1),
			new STextual.Coupler());
		return new STarget[]{valueTextual};
	}
	/**
	Re-implements abstract method. 
	@param definition is cast to a {@link TypedNode}, a path to the content
	root constructed and from this a {@link PathSelection} which
	 is passed to {@link #setSelection(SSelection)} PLUS...
	 */
	public void defineSelection(Object definition){
		TypedNode root=(TypedNode)framed;
		if(definition instanceof PathSelection){
			OffsetPath[]srcPaths=((PathSelection)definition).paths,
				nodePaths=new OffsetPath[srcPaths.length];
			for(int i=0;i<nodePaths.length;i++)
				nodePaths[i]=new NodePath(srcPaths[i].offsets);
			setSelection(new PathSelection(root,nodePaths));
			return;
		}
		else if(definition==SELECT_ALL){
			TypedNode[]descendants=Nodes.descendants(root);
			OffsetPath[]paths=new OffsetPath[descendants.length-1];
			for(int i=0;i<paths.length;i++)
				paths[i]=pathFromRoot(descendants[i+1]);
			setSelection(new PathSelection(root,paths));
			return;
		}
		TypedNode find=definition instanceof String?
				Nodes.descendantTitled(root,(String)definition)
				:definition instanceof Integer?root.children()[(Integer)definition]
				:(TypedNode)definition;
		if(false)trace(".defineSelection: ",find);
		List<TypedNode>descendants=Arrays.asList(Nodes.descendants(root));
		if(find==null)find=root;
		if(!descendants.contains(find))
			throw new IllegalStateException(Debug.info(definition)
					+" not selectable in "+Debug.info(this));
		for(TypedNode d:descendants)
			if(d==find){
				Object[]ancestry=Nodes.ancestry(find);
				setSelection(new PathSelection(ancestry[0],
						new OffsetPath[]{new NodePath(ancestry)}));
				break;
			}
		if(false)trace("~.defineSelection: ",selection());
	}
	protected void viewerSelectionChanged(SViewer viewer,SSelection selection){
		SSelection rootPaths=selection.content()instanceof SView?
				PathSelection.newMinimal(framed)
			:pathsFromRoot((PathSelection)selection);
		setSelection(rootPaths);
	}
	/**
	Converts viewer selections to full paths. 
	<p>Traces the path from the <code>indexed</code> of each path in 
	<code>selection</code> to the content root.  
	@param selection to be converted
	 */
	final protected PathSelection pathsFromRoot(PathSelection selection){
		OffsetPath[]rootPaths=new OffsetPath[selection.paths.length];
		Object content=selection.content();
		for(int i=0;i<rootPaths.length;i++){
			OffsetPath selectionPath=selection.paths[i];
			if(selectionPath==OffsetPath.empty){
				rootPaths[i]=selectionPath;
				continue;
			}
			NodePath nodePath=new NodePath(ancestry((DataNode)selectionPath.target(content)));
			if(selectionPath instanceof NodePath)
				nodePath=nodePath.valueAtChecked(((NodePath)selectionPath).valueAt());
			rootPaths[i]=nodePath;
		}
		return new PathSelection(framed,rootPaths);
	}
	public boolean actionIsLive(SViewer viewer,ViewableAction action){
		TypedNode root=(TypedNode)framed;
		Object[]selected=selection().selections();
		int arrayCount=root.children().length,selectionCount=selected.length;
		boolean empty=arrayCount==0,allSelected=arrayCount==selectionCount;
		TypedNode selectedNode=(TypedNode)selected[0];
		OffsetPath firstPath=((PathSelection)selection()).paths[0];
		if(!(firstPath instanceof NodePath))return false;
		int valueAt=((NodePath)firstPath).valueAt();
		return action==COPY||action==EDIT?root!=selectedNode
			:action==CUT||action==DELETE?root!=selectedNode||valueAt>=0
 			:action==PASTE||action==PASTE_INTO?canPaste()
			:action==ITERATE_FORWARD||action==ITERATE_BACK?
				!empty
			:action==SELECT_ALL?!empty&&!allSelected
			:super.actionIsLive(viewer,action);
	}
	@Override
	protected void actionTriggered(SViewer viewer,ViewableAction action){
		super.actionTriggered(viewer,action);
		if(action!=PASTE_INTO)return;
		pastingInto=true;
		insertStatefuls(newPasteStatefuls());
		updateAfterEditAction();
		pastingInto=false;
	}
	public void insertStatefuls(Stateful...statefuls){
		PathSelection paths=(PathSelection)selection();
		NodePath firstPath=(NodePath)paths.paths[0];
		int valueAt=firstPath.valueAt();
		TypedNode inserts[]=newTyped(TypedNode.class,statefuls),
			first=(TypedNode)selection().selections()[0],root=(TypedNode)framed;
		NodeList into=new NodeList(valueAt>=0||first==root||pastingInto?first
				:first.parent(),false);
		int firstAt=valueAt>=0?valueAt:into.indexOf(first);
		into.addAll(firstAt>=0?firstAt:0,inserts);
		if(!hasMixedContents(into.parent))into.updateParent();
		else into.updateMixedParent(first);
		setSelection(newNodeChangeSelection(root,inserts,false));
	}
	public void iterateSelection(boolean forward){
		TypedNode root=(TypedNode)framed;
		TypedNode[]descendants=Nodes.descendants(root);
		int markAt=-1,lastAt=descendants.length-1;
		Object selected[]=selection().selections(),
			mark=selected[forward?selected.length-1:0];
		if(mark!=root)for(int i=0;i<descendants.length;i++)
			if(descendants[i]==mark)markAt=i;
		markAt=markAt<1?forward?1:lastAt
				:forward?markAt==lastAt?1:markAt+1
				:markAt==1?lastAt:markAt-1;
		setSelection(PathSelection.newSingle(root,pathFromRoot(descendants[markAt])));
	}
	public void deleteSelection(){
		PathSelection paths=(PathSelection)selection();
		NodePath firstPath=(NodePath)paths.paths[0];
		int valueAt=firstPath.valueAt();
		if(valueAt>=0){
			ValueNode node=(ValueNode)firstPath.target(framed);
			int valueCount=node.values().length;
			node.deleteValueAt(valueAt);
			paths.paths[0]=firstPath.valueAtChecked(
					valueCount==1?-1:valueAt==valueCount-1?valueAt-1:valueAt);
			return;
		}
		TypedNode[]deletes=newTyped(TypedNode.class,paths.selections());
		List<TypedNode>reselects=new ArrayList();
		for(TypedNode delete:deletes){
			TypedNode parent=delete.parent();
			NodeList keeps=new NodeList(parent,false);
			keeps.clear();
			TypedNode[]thenChildren=parent.children();
			int thenAt=-1,nowAt=-1,thenLastAt=thenChildren.length-1;
			for(TypedNode thenChild:thenChildren){
				nowAt=++thenAt==thenLastAt?thenAt-1:thenAt+1;
				boolean keep=true;
				for(TypedNode check:deletes)keep&=(check!=thenChild);
				if(keep)keeps.add(thenChild);
				else reselects.add(thenChildren.length==1?parent:thenChildren[nowAt]);
			}
			if(!hasMixedContents(parent))keeps.updateParent();
			else keeps.updateMixedParent(null);
		}
		if(reselects.size()!=deletes.length)throw new IllegalStateException(
				"Selection lengths don't match "+Debug.info(this));
		setSelection(newNodeChangeSelection((TypedNode)framed,
				reselects.toArray(new TypedNode[]{}),true));
	}
	final protected PathSelection newNodeChangeSelection(TypedNode root, 
			TypedNode[]nodes,boolean removing){
		OffsetPath[]paths;
		if(false&&removing)
			paths=new OffsetPath[]{pathFromRoot(root)};
		else{
			paths=new OffsetPath[nodes.length];
			for(int i=0;i<paths.length;i++)paths[i]=pathFromRoot(nodes[i]);
		}
		return new PathSelection(root,paths);
	}
	public boolean textSeemsPastable(String text){
		return text.startsWith(XmlDocRoot.SHEBANG);
	}
	protected TypedNode[]newPasteTextStatefuls(String text){
		TreeRoot copyWrap=new XmlDocRoot(new DataNode(DataConstants.TYPE_XML,title(),
						new TypedNode[]{new ValueNode("Paste",title())}),new XmlPolicy());  
		try{
			copyWrap.readFromSource(Strings.stringLines(text));
		}catch(IOException e){
			throw new RuntimeException(e);
		}
		return copyWrap.tree.children()[0].children();
	}
	/**
	Re-implementation. 
	<p>Matches based on {@link TypedNode#type()} and {@link TypedNode#title()}.	
	 */
	protected Stateful[]matchSelectionEdits(Stateful[]targets,Stateful[]edits){
		Stateful[]matches=new Stateful[targets.length];
		for(int m=0;m<matches.length;m++){
			for(int i=0;i<edits.length;i++){
				TypedNode edit=(TypedNode)edits[i],target=((TypedNode)targets[m]);
				if(edit.type().equals(target.type())
						&&edit.title().equals(target.title())) 
								matches[m]=edits[i];
			}
			if(matches[m]==null)return null;
		}
		if(false)trace(".matchedTargetEdits: ",edits);
		return matches;
	}
	protected final TypedNode rootState(){
		if(rootState==null)throw new IllegalStateException(
				"Null rootState in "+Debug.info(this));
		return rootState;
	}
	private OffsetPath pathFromRoot(TypedNode node){
		return new NodePath(ancestry(node));
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

