package facets.core.app;
import facets.core.app.DialogSurface.ContentRoot;
import facets.core.superficial.SFacet;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.STarget;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SContentRootTargeter;
import facets.core.superficial.app.SContenter;
import facets.util.Debug;
import facets.util.Tracer;
import facets.util.Util;

import java.awt.Dimension;
/** 
{@link SContenter} that builds a content root with no 
viewers. 
<p>{@link SimpleContenter} is suitable for content that cannot 
  (or need not) appear in viewers, being exposed by an arrangement of 
  simple facets. 
<p>The content passed to the constructor is represented to the surface by 
targets returned by {@link #lazyContentRootElements()}.
<p>Since {@link SimpleContenter} primarily creates content for 
{@link DialogSurface}, it provides a basic implementation of 
{@link facets.core.app.DialogSurface.Contenter}.
 */
public abstract class SimpleContenter extends Tracer implements SContenter, 
		DialogSurface.Contenter{
	private final String title;
	private SFrameTarget contentFrame;
	/**
	Unique constructor. 
	@param title returned by {@link #title()}
	passed to {@link #lazyContentRootElements()}
	 */
	public SimpleContenter(String title){
		if(title==null||title.equals(""))throw new IllegalArgumentException(
				"Null or empty title in "+Debug.info(this));
		this.title=title;
	}
	/**
	Implements abstract method. 
	<p>Creates a trivial subclass of 
	{@link facets.core.app.DialogSurface.ContentRoot} 
	returning <code>elements</code> defined in 
	{@link #lazyContentRootElements()}, with (to meet the contract of 
	{@link SAreaTarget#activeContent()}) a single {@link SFrameTarget}
	child created from the content. 
	<p><b>Note</b> If the {@link SimpleContenter} is to be used in an 
	{@link AppSurface} this method must be re-implemented to return a
	comparable subclass of {@link facets.core.app.AppSurface.ContentRoot}.  
	 */
	public SAreaTarget newContentRoot(boolean faceted, boolean shareSelection){
		ContentRoot root=new DialogSurface.ContentRoot(title,
				new STarget[]{contentFrame=new SFrameTarget(title,Debug.info(SimpleContenter.this))},
				this){
			protected STarget[]lazyElements(){
				return lazyContentRootElements();
			}
		};
		if(faceted)attachRootMountFacet(root);
		return root;
	}
	/**
	Create simple targets representing the content.   
	 */
	protected abstract STarget[]lazyContentRootElements();
	/**
	Attach a facet to the tree headed by <code>root</code>. 
	<p>Called by {@link #newContentRoot(boolean, boolean)}; 
	the facet attached should be a suitable {@link MountFacet} for the 
	facet returned by {@link DialogSurface.Contenter#newContentPanel(SContentRootTargeter)}. 
	@param rootArea was created in {@link #newContentRoot(boolean, boolean)}
	 */
	protected abstract void attachRootMountFacet(SAreaTarget rootArea);
	/**
	Implements interface method. 
	<p>Empty impementation.  
	 */
	public void rootRetargeted(SContentRootTargeter t){}
	/**
	Implements abstract method. 
	<p>Returns the content passed to the constructor. 
	 */
	final public SFrameTarget contentFrame(){
		if(contentFrame==null)throw new IllegalStateException(
				"No contentFrame in "+Debug.info(this));
		return contentFrame;
	}
	/**
	Implements abstract method. 
	<p>Returns the title passed to the constructor. 
	 */
	final public String title(){return title;}
	/**
	Empty implementation. 
	 */
	@Override
	public void setDialog(DialogSurface dialog){}
	@Override
	public void reverseChanges(){}
	@Override
	public Dimension contentAreaSize(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	@Override
	public void applyChanges(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	@Override
	public void dialogClosed(){}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
