package facets.core.app;
import facets.core.superficial.SFacet;
import facets.core.superficial.STarget;
import facets.core.superficial.TargetCoupler;
import facets.core.superficial.app.SView;
/**
Can respond to requests from surface facets. 
<p>{@link SurfaceServices} defines useful functionality 
required by {@link SFacet}s from their enclosing surface. 
 */
public interface SurfaceServices{
	/**
	Return suitable facets for a viewer context menu.
	<p>The source of the request will generally be redundant as 
	the receiver will be able to determine which viewer is active;  
	in any case, other considerations such as the current selection will determine
	the facets to be returned.  
	 */
	MenuFacets getContextMenuFacets();
	/**
	Respond to an input to a facet which is invalid for its target. 
	<p>A simple facet will generally check validity by querying a 
	{@link TargetCoupler} attached to the target; a viewer facet by
	querying its {@link SView}.   
	@param target exposed by the facet
	@param input is known to be invalid 
	 */
	void handleInvalidInput(STarget target,Object input);
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

