package facets.core.app;
import facets.core.superficial.app.SSelection;
import facets.core.superficial.app.SViewer;
import facets.core.superficial.app.SelectionView;
import facets.util.Debug;
import facets.util.OffsetPath;
import facets.util.Stateful;
import facets.util.StatefulCore;
import facets.util.Titled;
import facets.util.Util;
import facets.util.datatree.ValueProxy;
import java.text.DateFormat;
import java.text.Format;
/**
{@link SelectionView} that generates tabular viewer data. 
 */
public abstract class TableView extends SelectionView{
	public TableView(String title){super(title);}
	/**
	Re-implementation that
<ol>
	<li>obtains a local  {@link PathSelection} by passing <code>viewable</code> to {@link #newTableSelection(PathSelection)}
	<li>obtains an array of data sources from {@link #getRowSources(PathSelection)}  
	<li>creates {@link ValueProxy} rows by 
	calling {@link #newRowProxy(Stateful)} for each source
	<li>returns a {@link PathSelection} of {@link ArrayPath}s created from the rows 
	and the local selection
	</ol>	
	 */
  @Override
	final public SSelection newViewerSelection(SViewer viewer,SSelection viewable){
		PathSelection table=newTableSelection((PathSelection)viewable);
		if(false)trace(".newViewerSelection1: ",table);
		Stateful[]rowSources=getRowSources(table);
		ValueProxy[]rows=new ValueProxy[rowSources.length];
		for(int i=0;i<rows.length;i++)rows[i]=newRowProxy(rowSources[i]);
		OffsetPath[]rowPaths=new ArrayPath[table.paths.length];
		for(int i=0;i<rowPaths.length;i++)rowPaths[i]=new ArrayPath(table.paths[i].offsets);
		return new PathSelection(rows,false?newRowArrayPaths(rows,viewable):rowPaths);
	}
	/**
	Return a local selection for use by {@link #newViewerSelection(SViewer, SSelection)}. 
	<p>Default implementation returns the {@link PathSelection} passed.
	@param viewable the {@link SSelection} passed to the calling method, 
	cast to a {@link PathSelection}
	 */
	protected PathSelection newTableSelection(PathSelection viewable){
		return viewable;
	}
	/**
	Return data sources for use by {@link #newViewerSelection(SViewer, SSelection)}. 
	<p>Default implementation casts the selection content to an array.
	@param table the {@link SSelection} passed to the calling method
	 */
	protected Stateful[]getRowSources(PathSelection table){
		return(Stateful[])table.content();
	}
	/**
	Create a {@link ValueProxy} that 
<ul>
<li>defines with {@link ValueProxy#valueCount()} the row length 
<li>supplies with {@link ValueProxy#get(int)} the values for each column
	</ul>	
	@param stateful will be the {@link ValueProxy#source} 
	 */
	protected abstract ValueProxy newRowProxy(Stateful stateful);
	/**
	Creates paths for return by {@link #newViewerSelection(SViewer, SSelection)}. 
	 @param rows created by the calling method
	 @param viewable was passed to the calling method
	 */
	private ArrayPath[]newRowArrayPaths(ValueProxy[]rows,SSelection viewable){
		OffsetPath[]paths=((PathSelection)viewable).paths;
		ArrayPath[]arrays=new ArrayPath[paths.length];
		for(int i=0;i<arrays.length;i++)arrays[i]=new ArrayPath(rows,paths[i]);
		return arrays;
	}
	public boolean allowMultipleSelection(){
		return false;
	}
	public boolean isColumnEditable(int col){
		return false;
	}
	public String getColumnTitle(int col){
		return "Column"+col;
	}
	public Format getColumnFormat(int col){
		return null;
	}
	public boolean allowSelectOnEdit(){
		return false;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

