package facets.core.app.avatar;
import facets.core.superficial.app.SViewer;
import facets.util.Debug;
import facets.util.Stateful;
import facets.util.geom.Point;
import facets.util.geom.Vector;
/**
Defines how a custom viewer should drag an avatar selection. 
<p>A {@link DragPolicy} supplies the {@link Painter}s needed by 
  a viewer to paint the content of an avatar selection as it 
  is dragged, and specifies the effects on content of terminating 
  the drag. 
 */
public abstract class DragPolicy{
	final public static Integer STATE_NORMAL=0,
		STATE_DRAG_UNCONSTRAINED=1, 
		CONSTRAINTS_NONE=0,CONSTRAINTS_CROSS=1,
		CONSTRAINTS_THREE_AXIAL=2,CONSTRAINTS_SIX_AXIAL=3;
	/**
 	Return motion painters for the specified drag position. 
  <p>New painters will generally be requested for each change 
    in <code>dragAt</code>. 
    @param anchorAt starting point of the drag 
	 * @param dragAt latest (possibly constrained) drag position 
	 * @return a non-<code>null</code> {@link Painter}[] 
	 */
	public abstract Painter[]newDragPainters(Point anchorAt,Point dragAt);
	/**
	Return suitable edits for the drop specified. 
    <p>@param anchorAt starting point of the drag 
    @param dragAt final (possibly constrained) drag position before dropping 
      @return an {@link Object}[] specifying changes to the content 
      of the dragged selection  
	 */
	public abstract Object[]newDragEdits(Point anchorAt,Point dragAt);
	/**
	Defines the cursor to appear over different drag states. 
    @param state an implementation-dependent object 
    @return an implementation-dependent object
	 */
	public Object stateCursor(Object state){
		return state==STATE_DRAG_UNCONSTRAINED?SViewer.CURSOR_MOVE:
				SViewer.CURSOR_DEFAULT;
	}
	/**
	Specifies constraints on the movement of the drag. 
	@return an implementation-dependent object
	 */
	public Object constraints(){return CONSTRAINTS_NONE;}
	/**
	Should the viewer attempt to snap the drag to non-selected 
    avatars?
	<p>Default returns <code>false</code>. 
	 */
	public boolean checkSnap(){return false;}
	/**
	Variant method for use in snapping. 
	<p>Invalid implementation
	 */
	public Object[]newSnapDropEdits(Point anchorAt,
			Point dragAt,Vector snapShift){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	/**
	Variant method for use in snapping. 
	<p>Invalid implementation
	 */
	public Painter[]newSnapPainters(Point anchorAt,
			Point dragAt,Vector snapShift){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
