package facets.core.app.avatar;	
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.SIndexing;
import facets.core.superficial.STarget;
import facets.core.superficial.TargetCore;
import facets.core.superficial.app.ViewerTarget;
import facets.util.Objects;
import facets.util.StatefulCore;
import facets.util.geom.Point;
import facets.util.geom.Vector;
/**
{@link PlaneViewWorks} that implements {@link ZoomPanView}.
<p>Also has a number of utility class methods.  
 */
public class ZoomPanViewWorks extends PlaneViewWorks implements ZoomPanView{
	final public static int MODEL_START=PlaneViewWorks.MODEL_ELEMENTS,
		MODEL_ZOOM=MODEL_START+0;
	private Point focusAt=new Point(0,0);
  private final StatefulCore state;
  public ZoomPanViewWorks(String title,double showWidth,double showHeight,
  		AvatarPolicies policies){
		super(title,showWidth,showHeight,new Vector(0,0),1.0, policies);
		state=new StatefulCore("State of "+title);
	}
  final public ResizeResponse resizeResponse(){
		return ResizeResponse.RESCALE;
	}
  final public Point focusAt(){return new Point(focusAt);}
  public STarget newFrame(){
    double showWidth=showWidth(),showHeight=showHeight();
    ZoomPanView[]zooms={
      new ZoomPanViewWorks("F&urthest 50%",showWidth/.5,showHeight/.5,avatars),
      new ZoomPanViewWorks("Far 75%",showWidth/.75,showHeight/.75,avatars),
      new ZoomPanViewWorks("Normal 100%",showWidth,showHeight,avatars),
      new ZoomPanViewWorks("N&ear 150%",showWidth/1.5,showHeight/1.5,avatars),
      new ZoomPanViewWorks("Nea&rest 200%",showWidth/2,showHeight/2,avatars),
    };
    STarget[]frames=ViewerTarget.newViewFrames(zooms);
    final STarget zoom=new SIndexing("Zoom",frames,0,
        new SIndexing.Coupler(){
      public boolean canCycle(SIndexing i){return false;}
      public String[]iterationTitles(SIndexing i){return new String[]{ZOOM_OUT,ZOOM_IN};}
      public void indexSet(SIndexing s)
      	{ZoomPanViewWorks.this.setState(((SFrameTarget)s.indexed()).framed);}      
    });
    final STarget[]allElements=TargetCore.join(
        super.newFrameElements(),new STarget[]{zoom});
    return new SFrameTarget(this){
      protected STarget[]lazyElements(){return allElements;}
    };
  }
	final public double originToLeft(){return -planeWidth()/2;}
	final public double originToTop(){return planeHeight()/2;}
  final public double planeHeight(){return 2;}
  final public double planeWidth(){return 2;}
  final public void setFocusAt(Point at)
  	{focusAt=new Point(at);state.updateStateStamp();}
  public void setState(Object state){
    super.setState(state);
		ZoomPanViewWorks src=(ZoomPanViewWorks)state;
    this.state.setState(src.state);
	}
  final public Object stamp(){return state.stateStamp();}
	protected AvatarPolicies fetchActorPolicies(){return avatars;}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

