package facets.core.superficial;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.OffsetPath;
import facets.util.Stateful;
import facets.util.Titled;
import facets.util.Util;
import facets.util.datatree.TypedNode;

import java.io.Serializable;
/**
Extends {@link TargetCore} by framing 
  application content to be exposed directly to facets. 
 */
public class SFrameTarget extends TargetCore{
	private static int frames;
	/**Immutable framed framed by the {@link SFrameTarget}.*/
	final public Object framed;
	/**
	Convenience constructor creating a suitable title.
	 <p>If <code>framed</code> is a {@link facets.util.Titled} its <code>title</code>
	  is used, therwise a dummy title is created.   
	 */
	public SFrameTarget(Object framed){
		this(framed instanceof Titled?((Titled)framed).title()
				:Util.helpfulClassName(framed)+"#"+frames++,framed);
	}
	/**
 	Core constructor. 
  <p>Note that this passes no child target elements to the superclass; 
    elements can only be set by subclassing and  
  <ul>
    <li>in named subclasses where the elements are known at 
      construction, calling {@link #setElements(STarget[])} 
      from the constructor 
    <li>in other cases (in practice the large majority), overriding 
      {@link #lazyElements()} from {@link TargetCore} 
  </ul>
  <p>This limitation ensures that the effective type of 
    a {@link SFrameTarget} with child elements can be distinguished 
    by reference to the compiled type. Care must therefore be 
    taken in applications not vary the effective type of the 
    elements created by a subclass. 
  @param title passed to the superclass 
  @param toFrame must not be <code>null</code>
	 */
	public SFrameTarget(String title,Object toFrame){
	  super(title);
		if((framed=toFrame)==null)throw new IllegalArgumentException(
				"Null framed in "+Debug.info(this));
	}
  /**
	Set the state of the framed. 
	<p>Default implementation is an invalid stub.  
	@param stateSpec must define the new state of {@link #framed}
	 @param interim if <code>true</code> the edit forms part of a sequence 
	 */
	public void setFramedState(Object stateSpec, boolean interim){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	protected TypedNode[]newDebugChildren(){
    ItemList<TypedNode>items=new ItemList(TypedNode.class);
    items.addItems(super.newDebugChildren());
    if(framed!=null)
      items.addItem(newDebugContentNode("framed",framed));
    return items.items();
  }
	protected final boolean notifiesTargeter(){
		return true;
	}
	public String title(){
		return framed==null||!(framed instanceof Titled)?super.title()
			:((Titled)framed).title();
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
