package facets.core.superficial;
import java.io.Serializable;
/**
Connects a facade {@link STarget} to the application. 
<p>While {@link TargetCoupler} itself defines no specific functionality, 
	the appropriate subtypes for each facade {@link facets.core.superficial.STarget} 
</p>
<ul>
	<li> define presentation policy of the {@link STarget} in the surface,</li>
	<li>enable the application to handle the {@link STarget}'s responses 
		to surface events. </li>
</ul>
<p> A {@link TargetCoupler} subtype instance can either be subclassed 
	as required (usually trivially) to meet application-specific requirements, 
	or simply provide default presentation policy and event handling. This 
	approach (rather than subclassing the facade {@link STarget}s themselves) 
	minimises coupling between application and framework classes, with the 
	further advantage that application code can often be simplified by sharing 
	a single coupler between logically linked targets. 
<p>The most fully worked out examples of this approach in Facets are the 
	extensive built-in policy options provided by {@link facets.core.superficial.SIndexing.Coupler}, 
	and the use of {@link facets.core.superficial.SNumeric.Coupler} to supply appropriate 
	{@link facets.util.NumberPolicy} instances to {@link facets.core.superficial.SNumeric}s. 
 
 */
public interface TargetCoupler extends Serializable{
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

