package facets.core.superficial.app;
import facets.core.superficial.Notifying;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.TargeterCore.TargetType;
import facets.util.Debug;
import facets.util.Util;
/**
Ancestor class of all targeters returned by {@link SAreaTarget}s. 
<p>{@link AreaTargeter} extends its superclass only in its behaviour when
retargeting facets.    
 */
public class AreaTargeter extends IndexingTargeter{
	/**
  Constructs an {@link AreaTargeter} to be retargeted on an 
  {@link SAreaTarget}.     
     */
  protected AreaTargeter(TargetType type){
  	super(type);
  }
  /**
	Extends superclass behaviour. 
	<p>Also calls {@link SAreaTarget#retargetFacets(Notifying.Impact)} in its target. 
	 */
	public void retargetFacets(Impact impact){
	  SAreaTarget areaTarget=(SAreaTarget)target();
	  if(areaTarget.parent()==null)areaTarget.retargetFacets(impact);
	  super.retargetFacets(impact);
	}
	/**
  Return the <code>indexedTargeter</code> at the specified depth. 
  @param depth
   */
  public final AreaTargeter area(int depth){
	  AreaTargeter area=this;
		boolean trace=false;
		if(trace)traceDebug(".area: ",area);
	  for(int i=0;i<depth;i++){
	  	if(trace)traceDebug(".area: depth="+i+" ",area);
	  	if(((IndexingTarget)area.target()).indexableTargets().length==0)break;
	  	STargeter below=area.indexedTargeter();
	  	if(below instanceof AreaTargeter)area=(AreaTargeter)below;
	  	else if(depth==SAreaTarget.AREA_LOWEST)break;
	  	else throw new IllegalArgumentException("Bad area depth " +depth+
	  			" in "+Debug.info(this));
	  }
	  if(trace)traceDebug(".~area: ",area);
	  return area;
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
