package facets.core.superficial.app;
import static facets.util.app.Events.*;
import facets.core.superficial.Notifying;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.TargetCore;
import facets.core.superficial.TargeterCore;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.TargeterCore.TargetType;
import facets.util.Debug;
import facets.util.ItemList;
import facets.util.Util;
import facets.util.app.Events;
import facets.util.datatree.TypedNode;

import java.util.HashMap;
import java.util.Map;
/**
FrameTargeter for an {@link IndexingTarget}.   
<p>{@link IndexingTargeter} defines much important behaviour used by 
	{@link facets.core.superficial.app.AreaTargeter}. In particular, it ensures 
	that at each retargeting an appropriate targeter is either created or 
	(more importantly) retrieved for retargeting to its target's indexed child; 
	the criterion used being the {@link TargetType} of the targeter returned by the 
	child. 
<p>Construction is package-private; this class is defined <code>public</code> 
for documentation purposes only. 
 */
public class IndexingTargeter extends TargeterCore{
  final private Map<Class<TargeterCore.TargetType>,STargeter>targeters=
  	new HashMap();
	private STargeter indexedTargeter,indexing;
  IndexingTargeter(TargetType type){super(type);}
  /**
  Retargeted to {@link IndexingTarget#indexedTarget()} in the current target.  
   */
  public final STargeter indexedTargeter(){
    if(indexedTargeter==null)throw new IllegalStateException("No childTargeter in "+Debug.info(this));
    return indexedTargeter;
  }
  /**
  Retargeted to {@link IndexingTarget#indexing()} in the current target. 
   */
  public final STargeter indexing(){
    if(indexing==null)throw new IllegalStateException(
    		"No indexingLink in "+Debug.info(this));
    return indexing;
  }
  /**
  Overrides superclass method. 
  <p>Creates and applies a targeter tree on the target tree headed
  by its {@link IndexingTarget} target. 
   */
  public void retarget(STarget target,Impact impact){
    super.retarget(target,impact);
    IndexingTarget it=(IndexingTarget)target;
    STarget ix=it.indexing(),
    	targets[]=it.indexableTargets(),
			indexed=it.indexedTarget();
    if(indexing==null)indexing=((TargetCore)ix).newTargeter();
    indexing.retarget(ix, impact);
    for(int i=0;i<targets.length;i++)
      if(targets[i]!=indexed&&targets[i]instanceof IndexingTarget)
        retargetedTargeter(targets[i],impact);
    indexedTargeter=retargetedTargeter(indexed,impact);
    it.indexing().setMonitor(it);
  }
  /**
  Overrides superclass method. 
  <p>Also calls {@link #retargetFacets(Notifying.Impact)} in {@link #indexing()}
  and {@link #indexedTargeter}. 
   */
  public void retargetFacets(Impact impact){
    super.retargetFacets(impact);
    STargeter indexed=indexedTargeter();
		indexed.retargetFacets(impact);
    indexing().retargetFacets(impact);
    if(true)for(STargeter t:targeters.values())if(t!=indexed)t.retargetFacets(impact);
  }
  /**
	Returns a targeter retargeted on the target passed. 
	<p>The key method in managing variation in target type.  
	@param target for targeter
   * @param impact 
	@return a targeter whose target type matches that of 
	{@link TargetCore#newTargeter()} in <code>target</code>; this will generally 
	have been stored during a previous retargeting. 
	 */
	final protected STargeter retargetedTargeter(STarget target,Impact impact){
		boolean debug=false||(Events.trace);
	  STargeter checkTargeter=((TargetCore)target).newTargeter();
	  TargeterCore.targeters--;
	  Class<TargetType>targetType=((TargeterCore)checkTargeter).targetType;
	  STargeter targeter=targeters.get(targetType);
	  if(targeter==null){
		  TargeterCore.targeters++;
	  	targeters.put(targetType,targeter=checkTargeter);
	    if(trace)traceEvent("Storing targeter for "+targetType+" in "+Debug.info(this));
	  }
	  else if(trace)Events.traceEvent("Retrieved targeter for "+targetType+" in "+Debug.info(this));
	  targeter.setMonitor(null);
	  if(false)traceDebug(".appliedTargeter: ",target);
	  targeter.retarget(target,impact);
	  targeter.setMonitor(this);
	  return targeter;
	}
	protected TypedNode[]newDebugChildren(){
    ItemList<TypedNode>items=new ItemList(TypedNode.class);
    items.addItems(super.newDebugChildren());
    items.addItem(newDebugContentNode("[targeters]",targeters.values().toArray()));
    if(indexedTargeter!=null)
      items.addItem(newDebugContentNode("indexedTargeter",indexedTargeter));
    if(indexing!=null)
      items.addItem(newDebugContentNode("indexingLink",indexing));
    return items.items();
  }
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

