package facets.core.superficial.app;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.TargeterCore.TargetType;
import facets.util.Titled;
/**
Builds a targeter tree and facets that can expose its content. 
<p>{@link SContenter} encapsulates creation of a targeter tree that  
exposes its content to user view and control, defining  
methods that can be called by a surface during (re)building.
<ul>
	<li>{@link #newContentRoot(boolean, boolean)} will return a content area root which 
		in turn creates the {@link facets.core.superficial.app.SContentRootTargeter} 
		to be used by the surface for the content type. 
  <li>{@link #rootRetargeted(SContentRootTargeter)} can be called after the initial 
  build and at each retargeting
</ul>
 */
public interface SContenter extends TargetType,Titled{
	/**
	Construct a content area tree. 
	<p>For use by {@link SSurface}s to create content area trees for incorporation
	into their surface area trees.  
	@param faceted should the area tree have facets attached?
	@param shareSelection should the new tree frame the existing viewable
	@return the {@link SAreaTarget} root of an area tree containing either
	the content or viewers for the content; the root should in turn return with
	{@link SAreaTarget#newTargeter()} a {@link SContentRootTargeter} for the content
	wrapped by the {@link SContenter}.
	 */
	SAreaTarget newContentRoot(boolean faceted, boolean shareSelection);
	/**
	Notify the contenter of retargeting. 
	<p>Should be called by the surface whenever its root {@link SContentRootTargeter}
	has been retargeted on a content root returned by {@link #newContentRoot(boolean, boolean)} 
	in this {@link SContenter}, with the following guarantees.
	<ul><li>Before the first invocation, all targeter tree elements exposing this content 
	will have been attached to <code>rootTargeter</code> - allows setting of
	references to other elements.
	<li>All attached facets will be retargeted after any invocation - allows
	setting of live states etc.
	</ul>
	@param t will be retargeted on a content root
	created by this {@link SContenter} type (not generally by this 
	instance). 
	 */
	void rootRetargeted(SContentRootTargeter t);
	/**
	Frame content wrapped by this contenter. 
	<p>Must be that returned by {@link SAreaTarget#activeContent()} in 
	each content root returned by {@link #newContentRoot(boolean, boolean)} in this
	{@link SContenter}. 
	 */
	SFrameTarget contentFrame();
	/**
	Return a suitable title for the content. 
	<p>This will generally be either the name of an external content source
	or a suitable dialog area title. 
	 */
	String title();
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

