package facets.core.superficial.app;
/**
GUI context for a surface. 
<p>{@link SHost} must be implemented by a facets builder 
  for each GUI context in which a surface is to be constructed. 
  <p>Such a context can comprise a single widget eg for a modal dialog, or 
  several eg the content area and menu, tool and status bars of a 
  complete desktop application. 
*/
public interface SHost{
	/**
	Arrangement of facets exposing application content. 
	<p>{@link FacetLayout} defines no contract more specific than that it can
	be passed to {@link SHost#setLayout(SHost.FacetLayout)},  
	but implementations can encapsulate specific layout options
	provided by the {@link SHost}. This approach also allows 
	complete layouts to be switched eg in multi-content applications. 
	 */
	public interface FacetLayout{
		FacetLayout HEADLESS=new FacetLayout(){};
	}
  /**
  The widget(s) wrapped by the {@link SHost}. 
    <p>Gives access to the actual widget(s) used by the GUI context 
      to attach the widget roots created by facet layouts.
   */
  Object wrapped();
	/**
	Attach widgets managed by the facets of the {@link FacetLayout}
	to the GUI context. 
	@param layout will generally have been created by the {@link SHost} 
	implementation
	 */
	void setLayout(FacetLayout layout);
	/**
	Update the layout last set and other features as specified by the application.  
	 */
	void updateLayout(SSurface surface);
	void setTitle(String title);
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

