package facets.core.superficial.app;
import facets.core.superficial.Notifiable;
import facets.util.Titled;
/**
Surface managing an {@link AreaTargeter} tree. 
<p>A {@link SSurface} should build a surface by 
<ol>
	<li>creating a targeter tree containing at least one 
	{@link SContentRootTargeter}
	<li>attaching facet to the targeter tree 
	<li>passing the facet to a {@link SHost} GUI context that can create
	the widget surface layer.
	<li>retargeting the targeter tree and its attached facet as required
</ol>
<p>This general contract can be implemented to build the surfaces of 
<ul>
	<li> simple applications or applets exposing a single content object
	<li> complex applications with multiple-content interfaces
	<li> modal dialogs with single or multiple content pages
</ul>
 */
public interface SSurface extends Notifiable,Titled{
	/**
  Build a targeter tree and pass its attached facet to a 
  GUI host.
	 */
	void buildRetargeted();
	/**
	The root of the surface targeter tree. 
	@return the root of the targeter tree created in {@link #buildRetargeted()}
	 */
	AreaTargeter surfaceRootTargeter();
	/**
	Return the host.
	<p>In an application lazily create a suitable 
	  host if required; otherwise return the host passed during 
	  construction.
	 */
	SHost host();
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
