package facets.core.superficial.app;
import facets.core.superficial.SFrameTarget;
import facets.util.Debug;
import facets.util.Titled;
import facets.util.Util;
/** 
{@link SFrameTarget} that maintains a {@link SSelection}.
<p>{@link SelectingFrame} extends its superclass by  
<ul><li>maintaining a {@link SSelection} defining a portion of its {@link #framed} content 
<li>allowing for the creation of {@link SFrameTarget}s framing the content 
selected.</ul>
 */
abstract public class SelectingFrame extends SFrameTarget{
	private SSelection selection;
  /**
  Unique constructor. 
  @param title passed to superclass
  @param content passed to superclass
   */
  public SelectingFrame(String title,Object content){
		super(title,content);
	}
	/**
	Define a selection within the content <code>framed</code>. 
	@param definition in the default implementation is cast to a {@link SSelection} 
	and passed to {@link #setSelection(SSelection)}
	 */
	public void defineSelection(Object definition){
		setSelection((SSelection)definition);		
	}
	/**
	The current selection within the content <code>framed</code>. 
	@return the {@link SSelection} last set with {@link #setSelection(SSelection)}
	 */
	final public SSelection selection(){
		if(selection==null)throw new IllegalStateException(
				"Null selection in "+Debug.info(this));
		return selection;
	}
  /**
	Sets the selection to be returned by {@link #selection()}. 
	<p>This method is defined protected as it should only be called by subclasses;
	external callers should use the more general {@link #defineSelection(Object)}.
	@param selection must have a {@link SSelection#content()} equal to
	{@link #framed}. 
	 */
	final protected void setSelection(SSelection selection){
		if(selection.content()!=framed)throw new IllegalArgumentException(
				"Bad selection content:\n" + selection.content()+
				"\nshould be "+framed+"\nin "+Debug.info(this));
		else this.selection=selection;		
		if(false)trace(".setSelection: ",selection.selections());
		if(false)Debug.printStackTrace(10);
	}
	/**
  Return a {@link SFrameTarget} framing the currently selected content. 
  <p>Content framed should be that returned in the <code>selection</code>
   methods of {@link #selection()}.
   <p>Default frames a single selection, titled from the selected content or 
   else using {@link #title()}. 
   */
	public SFrameTarget selectionFrame(){
		Object selected=selection().selection();
		return	selected instanceof Titled?new SFrameTarget(selected) 
			:new SFrameTarget(title(),selected);
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

