package facets.facet;
import static facets.core.app.PathSelection.*;
import static facets.facet.FacetFactory.*;
import facets.core.app.ActionViewerTarget;
import facets.core.app.NodePath;
import facets.core.app.NodeViewable;
import facets.core.app.PathSelection;
import facets.core.app.TreeView;
import facets.core.superficial.Notifiable;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.TargeterCore;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.AreaTargeter;
import facets.core.superficial.app.SContentRootTargeter;
import facets.core.superficial.app.SSelection;
import facets.core.superficial.app.SViewer;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.app.FacetAppValues;
import facets.facet.toolkit.KViewer;
import facets.facet.toolkit.KWrap;
import facets.facet.toolkit.Kit;
import facets.facet.toolkit.KitFacet;
import facets.util.Debug;
import facets.util.OffsetPath;
import facets.util.Tracer;
import facets.util.Util;
import facets.util.datatree.DataNode;
import facets.util.datatree.Nodes;
import facets.util.datatree.TypedNode;
import facets.util.datatree.ValueNode;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
/**
Displays a debug graph in the extras pane. 
<p>Constructed package-private in {@link FacetFactory#extras()}
 */
final class ExtrasGraph extends Tracer implements KitFacet{
	private final List<OffsetPath>findPaths=new ArrayList();
	private final String traceTop="Graph";
	private final DataNode paneRoot=new DataNode("Extras",traceTop);
	private final Kit kit;
	private final ValueNode stateApp;
	private final Class contentRootType;
	private final String title;
	private final String keyOffsets;
	private final NodeViewable viewable;
	private final ViewerTarget viewerTarget;
	private final KViewer viewer;
	private Object findThen;
	protected void traceOutput(String msg){
		System.out.println(traceTop+" "+title+msg);
	}
	ExtrasGraph(Kit kit,AreaTargeter targeter,final ValueNode stateApp){
		this.kit=kit;
		this.stateApp=stateApp;
		contentRootType=targeter.targetType;
		String typeKey=traceTop+Util.shortTypeNameKey(contentRootType);
		paneRoot.setTitle(title=typeKey);
		keyOffsets=FacetAppValues.KEY_GRAPH_OFFSETS+typeKey;
		paneRoot.setContents(newRootNode(targeter.target()));
		viewable=new NodeViewable(paneRoot){
			protected SSelection newViewerSelection(SViewer viewer){
				int[]in=((PathSelection)selection()).paths[0].offsets;
				TypedNode outRoot=paneRoot.children()[0];
				outRoot.setParent(null);
				PathSelection out=newSingle(outRoot,new NodePath(Arrays.copyOfRange(in,1,in.length)));
				if(false)trace(".newViewerSelection: out=",out);
				return out;
			};
			protected void viewerSelectionChanged(SViewer viewer,SSelection selection){
				int[]in=((PathSelection)selection).paths[0].offsets,out=new int[in.length+1];
				System.arraycopy(in,0,out,1,in.length);
				setSelection(PathSelection.newSingle(paneRoot,new NodePath(out)));
				stateApp.put(keyOffsets,out);
				if(false)trace(".viewerSelectionChanged: ",stateApp.getString(keyOffsets));
			}
		};
		OffsetPath startPath;
		try{
			int[]offsets=stateApp.getInts(keyOffsets);
			startPath=offsets.length>0?new NodePath(offsets):GRAPH_NO_PATH;
		}catch(Exception e){
			startPath=GRAPH_NO_PATH;
		}
		String find=graphFindNow?findGraphValue:GRAPH_FIND_NONE;
		boolean sameFind=find.equals(findThen);
		if(find.equals(GRAPH_FIND_NONE))findThen=null;
		else{
			if(!sameFind){
				findPaths.clear();
				graphFindAt=0;
			}
			for(TypedNode d:Nodes.descendants(paneRoot))
				if(d.title().toLowerCase().matches(".*\\Q"+find.toLowerCase()+"\\E.*"))
					findPaths.add(new NodePath(Nodes.ancestry(d)));
			findThen=find;
		}
		int findCount=findPaths.size();
		if(graphFindNow&&findCount>0){
			graphFindAt=findCount>graphFindAt?graphFindAt:0;
			if(false)trace(".refresh: find="+find+" sameFind="+sameFind+" finds="
					+findPaths.size()+" graphFindAt=",graphFindAt);
			stateApp.put(keyOffsets,(startPath=findPaths.get(graphFindAt++)).offsets);
		}
		if(false)trace(": startPath=",startPath.members(paneRoot));
		viewable.defineSelection(newSingle(paneRoot,startPath));
		TreeView view=new TreeView(title()){
			public String nodeRenderText(TypedNode node){
				return node.title();
			}
			public String contentIconKey(Object content){
				return null;
			}
		};
		viewerTarget=new ActionViewerTarget(title,viewable,
				new SFrameTarget(view)){
			public boolean isActive(){
				return true;
			}
		};
		viewer=kit.masteredViewer(this,
				new ViewerAreaMaster(){
			protected String hintString(){
				return HINT_EXTRAS_PANE+HINT_BARE;
			}
		},view,null);
	}
	public void retarget(STarget target,Impact impact){
		if(impact==Impact.FLASH)return;
		paneRoot.setContents(newRootNode(target));
		if(false)trace(".retarget: viewable=",viewable.selection());
		viewerTarget.retargetFacets(impact);
		((KViewer)base()).refresh(impact);
	}
	protected TypedNode newRootNode(STarget target){
		Notifiable check=target.monitor(),findRoot=check;
		while((check=((STargeter)check).monitor())instanceof STargeter){
			if(check instanceof SContentRootTargeter
					&&((TargeterCore)check).targetType!=contentRootType)
				throw new IllegalStateException("targetType!=contentRootType in "+Debug.info(this));;
			findRoot=check;
		}
		return((TargeterCore)findRoot).newGraphNode();
	}
	public KWrap base(){
		return viewer;
	}
	public String title(){
		return title;
	}
	public STarget target(){
		return viewerTarget;
	}
	public KWrap[]items(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public void targetNotify(Object notice,boolean interim){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
