package facets.facet;
import static facets.facet.FacetFactory.*;
import static facets.facet.HelpPages.*;
import facets.core.app.ActionViewerTarget;
import facets.core.app.AppActions;
import facets.core.app.NodePath;
import facets.core.app.TextTreeView;
import facets.core.app.TreeView;
import facets.core.superficial.FacetedTarget;
import facets.core.superficial.SFacet;
import facets.core.superficial.SFrameTarget;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SView;
import facets.facet.toolkit.KMount;
import facets.facet.toolkit.KWrap;
import facets.facet.toolkit.KitFacet;
import facets.util.Debug;
import facets.util.OffsetPath;
import facets.util.Tracer;
import facets.util.datatree.TypedNode;
import facets.util.datatree.ValueNode;
/**
Displays a {@link HelpPages} in the extras pane. 
<p>Constructed package-private in {@link FacetFactory#extras()}
 */
final class ExtrasHelp extends Tracer implements KitFacet{
	protected final static OffsetPath NO_PATH=new NodePath(new int[]{0,0});
	private final static String traceTop=ExtrasHelp.class.getSimpleName();
	private final HelpPages pages; 
	private final ValueNode stateApp;
	private final ViewerFacet tree,viewer;
	private final KitFacet area;
	private final KMount mount;
	protected void traceOutput(String msg){
		System.out.println(traceTop+msg);
	}
	ExtrasHelp(final HelpPages pages,ValueNode stateApp,FacetFactory ff){
		this.pages=pages;
		this.stateApp=stateApp;
		SView view=new TextTreeView(title()){
			protected String newPathNodeText(TypedNode[]path,String text){
				return pages.newPathNodeText(path,text);
			}
		};
		viewer=newViewerFacet(view,ff,false);
		area=(KitFacet)viewer.newAreaFacet(new SAreaTarget(title(),viewer.target));
		tree=true?null:newViewerFacet(view,ff,true);
		mount=tree==null?null:ff.kit.splitMount(this,false,null);
		if(mount!=null)mount.setItems(area.base(),tree.base());
	}
	private ViewerFacet newViewerFacet(SView view,final FacetFactory ff,
			final boolean tree){
		return new ViewerFacet(new ActionViewerTarget((tree?"Debug ":"")+title(),
				pages.viewable,
			tree?new SFrameTarget(new TreeView(title()){
				public boolean showRoot(){
					return true;
				}
			})
			:new SFrameTarget(view){
				protected STarget[]lazyElements(){
					return pages.targets;
				};
			}){
			public boolean isActive(){
				return true;
			}
		},
		ff.kit,
		new ViewerAreaMaster(){
			protected SFacet newViewerTools(STargeter t){
				STargeter[]elements=t.elements();
				return tree?super.newViewerTools(t)
					:ff.toolGroups(t,HINT_NONE, new SFacet[]{
							true?BREAK:ff.textualField(elements[TARGET_TOPIC],10,HINT_USAGE_FORM),
							ff.indexingIteratorButtons(elements[TARGET_HISTORY],HINT_BARE+HINT_TITLE2),
							ff.indexingIteratorButtons(elements[TARGET_FONT],HINT_BARE+HINT_TITLE2)});
			}
			protected String hintString(){
				return HINT_EXTRAS_PANE+HINT_NO_FOCUS+HINT_PANEL_ABOVE+HINT_PANEL_BORDER;
			}
		},
		null);
	}
	public void retarget(STarget target,Impact impact){
		if(!stateApp.getBoolean(AppActions.KEY_SHOW_EXTRAS))return;
		pages.updateFacet(this);
		((FacetedTarget)area.target()).retargetFacets(impact);
		if(tree==null)return;
		((FacetedTarget)tree.target()).retargetFacets(impact);
	}
	public KWrap base(){
		return mount!=null?mount:area.base();
	}
	public String title(){
		return "Help";
	}
	public STarget target(){
		return area.target();
	}
	public KWrap[]items(){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public void targetNotify(Object notice,boolean interim){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
