package facets.facet;
import facets.core.superficial.STarget;
import facets.core.superficial.Notifying.Impact;
import facets.facet.toolkit.Decoration;
import facets.facet.toolkit.KWidget;
import facets.facet.toolkit.KWrap;
import facets.facet.toolkit.KitFacet;
import facets.util.Debug;
import facets.util.StringFlags;
import facets.util.shade.Shade;
import java.util.HashMap;
import java.util.Map;
import javax.swing.JComponent;
/**
Builds and manages Swing panel for a simple facet. 
<p>{@link SwingSimpleMaster} descends from {@link facets.facet.FacetMaster.Simple}
via an internal superclass used by  the {@link FacetFactory} facet builder. 
<p>Custom facet can be created by passing concrete subclasses to
	 {@link facets.facet.FacetFactory#simpleMastered(facets.core.superficial.STargeter,facets.facet.FacetMaster.Simple)}.
*/
public abstract class SwingSimpleMaster extends SimpleMaster{
	private JComponent swingWrapped;
	public final Map<String,JComponent>swingMap=new HashMap();	
	public SwingSimpleMaster(){
		super(StringFlags.EMPTY);		
	}
	/**
	Create a panel facet base for attachment to the GUI. 
	@return a {@link JComponent} which may include others
	 */
	protected abstract JComponent newFacetBaseWrappable();
	/**
	The component returned by {@link #newFacetBaseWrappable()}. 
	 */
	final public JComponent facetBaseWrapped(){
		return(JComponent)core.base().wrapped();
	}
	@Override
	public abstract void retargetedSingle(STarget target,Impact impact);
	@Override
	public void retargetedMultiple(STarget[]targets,Impact impact){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	@Override
	final KWrap newBaseWrap(){
		swingWrapped=newFacetBaseWrappable();
		return new KWidget(){
			public Object wrapped(){
				return swingWrapped;
			}
			public KitFacet facet(){
				return core();
			}
			public Object newWrapped(Object parent){
				throw new RuntimeException("Not implemented in "+Debug.info(this));
			}
			public void setBackground(Shade color){
				throw new RuntimeException("Not implemented in "+Debug.info(this));				
			}
			public boolean hasFocus(){
				throw new RuntimeException("Not implemented in "+Debug.info(this));
			}
			public void redecorate(Decoration decorations){
				throw new RuntimeException("Not implemented in "+Debug.info(this));				
			}
			public void requestFocus(){
				throw new RuntimeException("Not implemented in "+Debug.info(this));				
			}
			public void setEnabled(boolean enabled){}
			public boolean isEnabled(){
				throw new RuntimeException("Not implemented in "+Debug.info(this));
			}
			public void setIndeterminate(boolean on){
				throw new RuntimeException("Not implemented in "+Debug.info(this));				
			}
			public Map<String,?> components(){
				return swingMap;
			}
		};
	}
	final KWrap[]newPartWraps(){
		return null;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

