package facets.facet;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.toolkit.KitFacet;
import facets.facet.toolkit.swing.ViewerMaster;

import javax.swing.JComponent;
/**
Builds and manages Swing pane for a viewer facet. 
<p>{@link SwingViewerMaster} descends from {@link facets.facet.FacetMaster.Viewer}
via a superclass known by the {@link AreaFacets} viewers builder. 
<p>A custom viewer facet can be defined by returning a concrete subclass from
{@link facets.facet.ViewerAreaMaster#viewerMaster()}. 
 */
abstract public class SwingViewerMaster extends ViewerMaster{	
	/**
	{@link SwingViewerMaster} defined by a {@link ViewerPaneForm}.
	 */
	public static class Form extends SwingViewerMaster{
		private final ViewerPaneForm form;
		public Form(ViewerPaneForm form){
			this.form=form;
		}
		@Override
		protected JComponent newAvatarPane(){
			form.attachPaneMaster(this);
			return (JComponent)((KitFacet)form.newRetargetedFormFacet()).base().wrapped();
		}
		@Override
		public void refreshAvatars(Impact impact){
			form.refreshFacets();
		}
	}
	/**
	Create the avatar pane to be used in the viewer facet.	
	<p>Called during viewer construction; the pane returned can be accessed
	with {@link #avatarPane()}. 
	 */
	protected abstract JComponent newAvatarPane();
	/**
	The Swing avatar pane. 
	@return the pane created in {@link #newAvatarPane()}
	 */
	final public JComponent avatarPane(){
		return super.avatarPane();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

