package facets.facet.app;
import static facets.facet.FacetFactory.*;
import static facets.util.Debug.*;
import facets.core.app.AppConstants;
import facets.core.app.DialogHost;
import facets.core.app.DialogSurface;
import facets.core.app.Dialogs;
import facets.core.app.ListView;
import facets.core.app.TreeView;
import facets.core.app.Dialogs.Surfacer;
import facets.core.superficial.Notice;
import facets.core.superficial.SFacet;
import facets.core.superficial.STarget;
import facets.core.superficial.STargeter;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.AreaTargeter;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.SHost;
import facets.core.superficial.app.SSurface;
import facets.core.superficial.app.SView;
import facets.facet.AreaFacets;
import facets.facet.FacetFactory;
import facets.facet.ViewerAreaMaster;
import facets.util.Debug;
import facets.util.Util;
import facets.util.app.AppValues;
import facets.util.datatree.DataConstants;
import facets.util.datatree.NodeList;
import facets.util.datatree.TypedNode;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.List;
/**
{@link DialogSurface} with attached {@link FacetFactory}. 
<p>Declared <code>abstract</code> to enable use of classname in values keys. 
 */
public abstract class FacetDialogSurface extends DialogSurface{
	/**
	{@link FacetDialogSurface} for multi-page dialogs. 
	 */
	public abstract static class MultiPaged extends FacetDialogSurface{
		public MultiPaged(FacetAppSurface app,DialogHost host,String title,
				DialogActions actions,Contenter[]contents){
			super(app,host,title,actions,contents);
		}
		final protected boolean isMultiPage(){return true;}
		final protected void attachSurfaceRootFacets(SAreaTarget areaRoot){
			STarget[]children=areaRoot.indexableTargets();
			AreaFacets areas=ff.areas();
			SFacet[]childrenFacets={
				areas.viewerArea((SAreaTarget)children[MULTI_AREA_TREE],
						new ViewerAreaMaster(){
					protected String hintString(){
						return HINT_BARE;
					}
				}),
				areas.switchMount((SAreaTarget)children[MULTI_AREA_PAGES]),
			};
			double ratio=(double)TRIMMINGS_TREE/(double)getLaunchBounds(areaRoot).width;
			areas.panes(areaRoot,childrenFacets,
					Util.shortTypeNameKey(this),HINT_NONE, 
					new int[]{AreaFacets.PANE_SPLIT_VERTICAL},
					new double[]{ratio},new int[]{1,1});
		}
		/**
		Re-implementation creating either a {@link ListView} or a {@link TreeView}. 
		 */
		final protected SView newPagesTreeView(TypedNode[]treeNodes){
			boolean fullTreeView=false;
			for(TypedNode node:treeNodes)fullTreeView|=node.children().length>0;
			return !fullTreeView?new ListView(title()){
				public boolean useTextFont(){
					return super.useTextFont();
				}
				public String nodeRenderText(TypedNode node){
					return node.title();
				}
				protected boolean includeValue(TypedNode parent,Object value){
					return false;
				}
			}:new TreeView(title()){
				public String contentIconKey(Object content){
					return DataConstants.NO_ICON;
				}
				public boolean showRoot(){
					return false;
				}
				public boolean canChangeSelection(){
					return true;
				}
				protected boolean filterNodeContents(){
					return true;
				}
				protected boolean includeValue(TypedNode parent,Object value){
					return false;
				}
				public String nodeRenderText(TypedNode node){
					return node.title();
				}
			};
		}
		protected SAreaTarget[]newContentPages(Contenter[] contents, 
				SAreaTarget[]contentRoots){
			return contentRoots;
		}
	}
	protected final FacetFactory ff;
	final boolean debugResizable=AppValues.stateDebug.getBoolean(Dialogs.DEBUG_RESIZABLE);
	private SFacet extension;
	private boolean extendSideways;
	public FacetDialogSurface(FacetAppSurface app,DialogHost host,String title, 
			DialogActions actions,Contenter[]contents){
		super(app,host,title,actions,contents);
		this.ff=app.ff;
	}
	protected void attachSurfaceRootFacets(SAreaTarget root){
		ff.areas().switchMount(root);
		if(false)trace(".attachSurfaceRootFacets: ",root.indexableTargets());
	}
	final protected void attachPageFacet(SAreaTarget panel){
		ff.areas().tabs(panel,HINT_NONE);
	}
	final protected SFacet newControlButtons(STargeter link){
		return ff.triggerButtons(link,(false?HINT_DEBUG:"")+
				HINT_USAGE_PANEL+HINT_GRID+HINT_BARE);
	}
	final public Rectangle scaledLaunchBounds(double factor){
		Rectangle bounds=getLaunchBounds((SAreaTarget)surfaceRootTargeter().target());
		Point at=bounds.getLocation();
		if(at.x==AT_NOT_SET)at=calculateSmartDialogAt(bounds.getSize(),app.host());
		return new Rectangle(at,scaleDimension(bounds.getSize(),factor,false));
	}
	final public void hostWindowDismissed(Rectangle closingBounds,double factor){
		for(Contenter content:contents())content.dialogClosed();
		hostWindowDismissed(new Rectangle(closingBounds.getLocation(),
					scaleDimension(closingBounds.getSize(),factor,true)));
	}
	private static Dimension scaleDimension(Dimension size,double factor,
			boolean invert){
		if(invert)factor=1/factor;
		return new Dimension((int)(size.width*factor),(int)(size.height*factor));
	}
	public boolean isResizable(){
		return persistBoundsState()||debugResizable;
	}
	public void addExtensionPanel(SFacet panel,boolean atSide){
		if(extension!=null)throw new IllegalArgumentException(
				"Already added "+info(panel)+" in " +info(this));
		else extension=panel;
		extendSideways=atSide;
	}
	/**
	Overrides superclass method. 
	<p>{@link Notice}s with {@link Impact#FLASH} are ignored
	 */
	public void notify(Notice notice){
		if(notice==null)throw new IllegalArgumentException("Null notice in "+Debug.info(this));
		else if(notice.impact==Impact.FLASH)return;
		if(extension!=null){
			if(false)trace(".notify: extension=",extension.target());
			((DialogHost)host()).setWindowExtension(extension.target().isLive()?
					extension:null,extendSideways);
		}
		super.notify(notice);
	}
	protected SFacet newExtras(AreaTargeter targeter){
		return ff.extras(targeter);
	}
	protected void traceOutput(String msg){
		if(false)super.traceOutput(msg);
	}
	/**
	Convenience method for creating a tabbed dialog. 
	@return {@link Surfacer} that will created an appropriate {@link FacetDialogSurface}
	 */
	public static Surfacer newDefaultTabbedSurfacer(){
		return new Surfacer(){
			protected DialogSurface newSurface(SSurface parent,DialogHost host,String title,
					DialogActions actions,Contenter[]contents){
				return new FacetDialogSurface((FacetAppSurface)parent,host,title,actions,contents){};
			}
		};
	}
	/**
	Convenience method for creating a paged dialog controlled by a tree or list view. 
	@return {@link Surfacer} that will create a {@link FacetDialogSurface} with a private
	implementation of {@link FacetDialogSurface#newPageTreeNodes(SAreaTarget[])} creating an
	appropriate tree. 
	@param apps contenters creating pages to appear directly under the tree root
	@param debugs contenters creating pages to appear under a child root created 
	from the first member which must be titled {@link FacetPreferences#TITLE_HEADER}   
	 */
	public static Surfacer newDefaultPagedSurfacer(final Contenter[]apps,
			final Contenter[]debugs){
		return new Surfacer(){
			protected DialogSurface newSurface(SSurface parent,
					DialogHost host,String title,DialogActions actions,Contenter[]contents){
				return new MultiPaged((FacetAppSurface)parent,host,title,actions,contents){
		protected TypedNode[]newPageTreeNodes(SAreaTarget[]pages){
			if(true)throw new RuntimeException("Not implemented in "+Debug.info(this));
			List<TypedNode>nodes=new ArrayList();
			SAreaTarget debugHeader=pages[apps.length];
			if(!debugHeader.title().equals(FacetPreferences.TITLE_HEADER))
				throw new IllegalStateException("Bad header title in "+debugHeader);
			NodeList debugRoot=debugs.length==0?null
				:new NodeList(newPageTreeNode(debugHeader),true);
			for(SAreaTarget page:pages){
				if(page.title().equals(FacetPreferences.TITLE_HEADER))continue;
				TypedNode node=newPageTreeNode(page);
				cleanNodeTitle(node);
				if(debugRoot!=null&&page.title().startsWith(FacetPreferences.TITLE_START))
					debugRoot.add(node);
				else nodes.add(node);
			}
			if(debugRoot!=null){
				cleanNodeTitle(debugRoot.parent);
				nodes.add(debugRoot.parent);
			}
			return nodes.toArray(new TypedNode[]{});
		}
		private void cleanNodeTitle(TypedNode node){
			node.setTitle(node.title().replace(FacetPreferences.TITLE_START,""));
		}
	};
			}
		};
	}
	public static Point calculateSmartDialogAt(Dimension box,SHost host){
		WindowAppHost appHost=(WindowAppHost)host;
		Dimension screen=appHost.screenSize();
		double screenRatio=screen.width/(double)screen.height;
	  Rectangle bounds=appHost.windowBounds();
	  if(bounds.width==0)bounds=new Rectangle(screen);
	  if(bounds.x+bounds.width>screen.width)screen.width*=2;
	  int screenX=screen.width/2,hostX=bounds.x+bounds.width/2,
	  	atX=screenX+(hostX-screenX)/2-box.width/2,
	  	screenY=screen.height/2,hostY=bounds.y+bounds.height/2,
	  	atY=screenY+(hostY-screenY)/2-box.height/2;
	  Point smartAt=new Point(atX,atY);
		return smartAt;
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
