package facets.facet.app.tree;
import static facets.facet.app.tree.TreeTargets.*;
import facets.core.app.MenuFacets;
import facets.core.app.SurfaceServices;
import facets.core.app.ViewerContenter;
import facets.core.superficial.SFacet;
import facets.core.superficial.STargeter;
import facets.core.superficial.app.SContentRootTargeter;
import facets.core.superficial.app.SHost;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.FacetFactory;
import facets.facet.app.FacetAppSpecifier;
import facets.facet.app.FacetAppSurface;
/**
{@link FacetFactory} for use with {@link TreeAppContenter}. 
<p>Like {@link TreeAppContenter}, effectively a package-private class 
exemplifying specialisation of {@link FacetFactory} 
to match a {@link FacetAppSpecifier} and {@link ViewerContenter}.  
 */
final public class TreeAppFacets extends FacetFactory{
	private final FacetAppSurface app;
	private final SContentRootTargeter root;
	/**
	Called from {@link TreeAppContenter#newContentLayout(SHost, SFacet, SContentRootTargeter)}. 
	@param app created in {@link TreeAppSpecifier#newApp(FacetFactory)}
	@param root as returned by {@link FacetAppSurface#activeContentRootTargeter()}
	 */
	protected TreeAppFacets(FacetAppSurface app,SContentRootTargeter root){
		super(app.ff);
		this.app=app;
		this.root=root;
	}
	/**
	Calls {@link #newAdjustedMenus(FacetAppSurface, SContentRootTargeter)} to add 
	tree menu. 
	@return available standard menus with interpolated tree menu
	 */
	@Override
	public SFacet[]menus(){
		return newAdjustedMenus(app,root);
	}
	/**
	Re-implementation based on {@link TreeAppContenter}. 
	 */
	@Override
	protected STargeter findPaneTargeter(SContentRootTargeter root){
		return root.elements()[TreeAppContenter.TARGETS_PANE];
	}
	/**
	Adds tree menu.
	@return available standard menus with interpolated menu containing
	the return of {@link TreeAppSpecifier#newTreeMenuItems(FacetFactory, STargeter[], 
	STargeter[]) }
	 */
	@Override
	protected MenuFacets[]adjustMenuRoots(MenuFacets[]menus){
		STargeter tree=root.content();
		final SFacet[]treeItems=((TreeAppSpecifier)app.spec).newTreeMenuItems(this,
				tree.elements(),root.elements()[TreeAppContenter.TARGETS_CONTENT].elements()
			);
		return new MenuFacets[]{
			menus[MENU_APP],
			menus[MENU_EDIT],
			menus[MENU_PANE],
			new MenuFacets(TreeTargets.TITLE_MENU,tree){
				@Override
				public SFacet[]getFacets(){
					return treeItems;
				}
			},
			menus[MENU_WINDOW],
			menus[MENU_HELP],
		};
	}
/**
	Creates edit/save/restore toolbar. 
	 */
	@Override
	public SFacet toolbar(){
		SFacet[]editTools=editTools(root);
		if(!app.spec.canEditContent())return null;
		AppTargeters app=new AppTargeters(root,this.app.spec);
		return toolGroups(root,HINT_NONE,app.file==null?editTools
				:join(new SFacet[]{
					triggerButtons(app.fileSave,HINT_BARE),
					triggerButtons(app.fileRevert,HINT_BARE),
					BREAK},
					editTools));
	}
	/**
	Creates specialised {@link MenuFacets} to match {@link TreeAppSpecifier}. 
	<p>Edit facets are appended where appropriate to tree expansion sub-menu.  
 */
	@Override
	public SurfaceServices services(){
		MenuFacets context=new MenuFacets("Tree facets",root){
			SFacet[]editFacets=new EditFacets(root).getFacets();
			public SFacet[]getContextFacets(ViewerTarget viewer,SFacet[]viewerFacets){
				return editFacets.length==0?viewerFacets
						:join(viewerFacets,join(new SFacet[]{BREAK},editFacets));
			}
		};
		return app.newSimpleServices(context);
	}
	/**
	Final implementation setting diagnostic status display. 
	 */
	@Override
	public SFacet status(){
		STargeter targeter=root.selection().elements()[0];
		return toolGroups(targeter,HINT_NONE,spacerTall(15),
			textualLabel(targeter,HINT_NONE));
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
