package facets.facet.toolkit;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import javax.swing.DefaultListCellRenderer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
final class JarIcons{
	final class JarPath{
		final String path;
		final boolean looksDisabled;
		JarPath(String project,String name){
			this.path=(project+"/"+name.replace(".gif",""));
			String[]members=path.split("/");
			looksDisabled=members[members.length-2].startsWith("d");
		}
		String getName(){
			String[]members=path.split("/");
			return members[members.length-1];
		}
		@Override
		public String toString(){
			return path;
		}
		@Override
		public boolean equals(Object that){
			return path.equals(((JarPath)that).path);
		}
		@Override
		public int hashCode(){
			return path.hashCode();
		}
	}
	final class ByteGif implements Comparable<ByteGif>{
		final Set<JarPath>paths=new HashSet();
		final Set<String>names=new HashSet();
		final byte[]content;
		final Icon icon;
		ByteGif(byte[]content){
			this.content=content;
			icon=new ImageIcon(content);
		}
		JLabel newLabel(){
			return new JLabel(toString(),new ImageIcon(content),0);
		}
		@Override
		public String toString(){
			return paths.size()+": "+
				paths.toArray()[0].toString().replaceAll(".*/([^/]+)$","$1");
		}
		@Override
		public boolean equals(Object that){
			return Arrays.equals(content,((ByteGif)that).content);
		}
		@Override
		public int hashCode(){
			return Arrays.hashCode(content);
		}
		@Override
		public int compareTo(ByteGif that){
			return that.paths.size()-paths.size();
		}
		void addPath(JarPath path){
			if(paths.size()>0)trace("Duplicate content from path="+path);
			paths.add(path);
			names.add(path.getName());
		}
	}
	private void trace(String msg){
		if(false)System.out.println(msg);
	}
	private JPanel newByteGifsList(boolean disables){
		Collection<ByteGif>values=(disables?disableds:enableds).values();
		List sort=new ArrayList(values);
		Collections.sort(sort);
		JPanel frame=new JPanel(new BorderLayout(0,5));
		frame.add(new JLabel((disables?"Disabled":"Enabled")+": "+
				values.size()),BorderLayout.NORTH);
		JList list=new JList(sort.toArray());
		list.setLayoutOrientation(JList.VERTICAL);
		list.setVisibleRowCount(20);
		list.setCellRenderer(new DefaultListCellRenderer(){
			private final JLabel label=new JLabel();
			@Override
			public Component getListCellRendererComponent(JList list,Object value,
					int index,boolean isSelected,boolean cellHasFocus){
				ByteGif gif=(ByteGif)value;
				label.setIcon(gif.icon);
				label.setText(gif.paths.size()+": "+gif.names.toString());
				return label;
			}
			
		});
		frame.add(new JScrollPane(list),BorderLayout.CENTER);
		return frame; 
	}
	final Map<ByteGif,ByteGif>enableds=new HashMap(),disableds=new HashMap();
	private final String includes;
	JarIcons(Collection<File>files,String includes,String excludes){
		this.includes=includes;
		for(File file:files){
			String fileName=file.getName();
			trace("File="+file);
			if(fileName.endsWith(".jar"))try{
				ZipFile zip=new ZipFile(file);
				String project=fileName.replaceAll("_.*","");
				for(ZipEntry entry:Collections.list(zip.entries())){
					String name=entry.getName();
					if(!name.endsWith(".gif"))continue;
					boolean skip=false;
					if(skip=!includes.trim().equals(""))
						for(String split:includes.split(","))
							if(!(skip=name.indexOf(split.trim())<0))break;
					if(skip)continue;
					for(String split:excludes.split(","))
						if((skip=!split.equals("")&&name.indexOf(split.trim())>=0))break;
					if(skip)continue;
					byte content[]=new byte[(int)entry.getSize()];
					InputStream stream=zip.getInputStream(entry);
					for(int i=0;i<content.length;i++)content[i]=(byte)stream.read();
					JarPath path=new JarPath(project,name);
					Map<ByteGif,ByteGif>checkGifs=path.looksDisabled?disableds:enableds;
					ByteGif gifKey=new ByteGif(content);
					if(gifKey.icon.getIconWidth()!=16)continue;
					ByteGif checkGif=checkGifs.get(gifKey);
					if(checkGif==null)checkGifs.put(gifKey,checkGif=gifKey);
					checkGif.addPath(path);
				}
			}catch(Exception e){
				throw new RuntimeException(e);
			}
		}
	}
	void showMatches(){
		JPanel message=new JPanel(new GridLayout(1,2,10,0));
		message.setPreferredSize(new Dimension(600,400));
		message.add(newByteGifsList(false));
		message.add(newByteGifsList(true));
		JOptionPane.showMessageDialog(null,message,
				JarIcons.class.getSimpleName()+" - '"+includes+"'",
				JOptionPane.PLAIN_MESSAGE);
	}
	static void main(String[]args){
		Set<File>files=new HashSet();
		if(true)files.add(new File("C:/eclipse/configuration/" +
				"org.eclipse.osgi/bundles/366/1/.cp/icons.jar"));
		else files.addAll(Arrays.asList(new File("C:/eclipse/plugins").listFiles()));
		new JarIcons(files,"_co",
			"birt,datatools,dsdp,e4,incubator,modeling,technology,archive,to_be_deleted," +
			"tptp,.test,webtools"
		).showMatches();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

