package facets.facet.toolkit.avatar;
import facets.core.app.avatar.AvatarContent;
import facets.core.app.avatar.AvatarView;
import facets.core.app.avatar.DragPolicy;
import facets.core.app.avatar.PainterSource;
import facets.core.superficial.app.SViewer;
import facets.util.Debug;
import facets.util.geom.Point;
import facets.util.geom.Vector;
import java.awt.event.MouseEvent;
final class Drag{
	final Avatar[]avatars;
	final private Point anchorAt;
	final private DragConstraints constraints;
	final private AvatarContent[]content;
	final private DragPolicy policy;
	private final boolean doSnap;
	private Point thenSnapped;
	final boolean hasPolicy;
	Drag(AvatarView view,Avatar[]avatars,Point anchorAt,Object hit, 
			PainterSource hostPainters){
		this.avatars=avatars;
		this.anchorAt=anchorAt;
		content=new AvatarContent[avatars.length];
		for(int i=0;i<content.length;i++)
			content[i]=avatars[i].content;
		policy=view.avatars().dragPolicy(view,content,hit,hostPainters);
		hasPolicy=policy!=null;
		if(!hasPolicy){
			doSnap=false;constraints=null;
			return;
		}
		doSnap=policy.checkSnap();
		int type=(Integer)policy.constraints(),gap=10;
		constraints=type==DragPolicy.CONSTRAINTS_CROSS?
				DragConstraints.newCross(gap)
			:type==DragPolicy.CONSTRAINTS_THREE_AXIAL?
					DragConstraints.newThreeAxial(gap)
			:type==DragPolicy.CONSTRAINTS_SIX_AXIAL?
					DragConstraints.newSixAxial(gap)
						:null;
	}
	public Object[]respondToMouse(AvatarCanvas canvas,Point mouseAt,int eventVals, 
			PainterSource painters){
		Point dragAt=constraints==null?mouseAt:
			constraints.constrain(anchorAt,mouseAt);
		Vector snapShift=null;
		if(doSnap){
			PlaneCanvas plane=(PlaneCanvas)canvas;
			double snapGap=plane.markLeap();
			snapShift=plane.checkSnap(dragAt,snapGap);
			if(snapShift!=null&&thenSnapped==null)
				thenSnapped=new Point(dragAt.at().plus(snapShift));
			else if(thenSnapped!=null){
				double reachCheck=thenSnapped.jumpFrom(dragAt).reach()-snapGap;
				if(reachCheck>0)thenSnapped=null;
				else snapShift=new Vector(0,0);
			}		
		}
		switch(eventVals&0xFFF){
		case MouseEvent.MOUSE_DRAGGED:
			canvas.host.setCanvasCursor(constraints==null?(Integer)
					policy.stateCursor(DragPolicy.STATE_DRAG_UNCONSTRAINED):
						constraints.dragCursor());
			return doSnap?policy.newSnapPainters(anchorAt,dragAt,snapShift)
				:policy.newDragPainters(anchorAt,dragAt);
		case MouseEvent.MOUSE_RELEASED:
			canvas.host.setCanvasCursor(SViewer.CURSOR_DEFAULT);
		case AvatarCanvas.MOUSE_DRAG_NOTIFY:
			return mouseAt.equals(anchorAt)?null:doSnap?
					policy.newSnapDropEdits(anchorAt,dragAt,snapShift)
					:policy.newDragEdits(anchorAt,dragAt);
		default:
			throw new IllegalArgumentException("Bad event values in "+Debug.info(this));
		}
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
