package facets.facet.toolkit.avatar;
import static facets.core.app.avatar.PlaneView.ResizeResponse.*;
import facets.core.app.avatar.PainterSource;
import facets.core.app.avatar.PlaneView;
import facets.core.app.avatar.ZoomPanView;
import facets.core.app.avatar.PlaneView.ResizeResponse;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.SViewer;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.toolkit.KViewer;
import facets.facet.toolkit.KitFacet;
import facets.util.Debug;
import facets.util.Doubles;
import facets.util.StringFlags;
import facets.util.Times;
import facets.util.Util;
import facets.util.geom.Line;
import facets.util.geom.Point;
import facets.util.geom.Vector;
import facets.util.shade.Shade;
class PlaneCanvas extends AvatarCanvas{
	interface PlaneHost extends CanvasHost{
		void adjustToPlane(int planeWidth,int planeHeight,int viewerLeft,
				int viewerTop);
	}
	protected Point origin=new Point();
	protected double scale=Double.NaN;
	int paneWidth,paneHeight;
	private boolean initialPortAtZeros=true;
	private PlaneView view;
	private Object stamp="No stamp";
	private ResizeResponse resize;
	PlaneCanvas(KitFacet facet,StringFlags hints){
		super(facet,hints);
	}
	protected void refreshViewPainters(){
		PlaneView viewNow=planeView();
		Object stampNow=viewNow.stamp();
		if(!viewNow.equals(view)||!stampNow.equals(stamp)){
			view=viewNow;
			stamp=stampNow;
			adjustToPaneSize(paneWidth,paneHeight);
		}
		super.refreshViewPainters();
	}
	protected void paneSizeSet(int width,int height){
		if(width<0||height<0)throw new IllegalArgumentException(
				"Negative dimensions in "+Debug.info(this));
		adjustToPaneSize(width,height);
	}
	private void adjustToPaneSize(int paneWidth,int paneHeight){
		if(view==null&&paneHeight!=0){
			this.paneWidth=paneWidth;
			this.paneHeight=paneHeight;
		}
		if(paneHeight==0||view==null)return;
		double showWidth=view.showWidth(),showHeight=view.showHeight();
		Vector plotShift=view.plotShift();
		resize=view.resizeResponse();
		if(scale!=scale||resize==RESCALE){
			scale=showWidth/showHeight>(float)paneWidth/(float)paneHeight?
					paneWidth/showWidth:paneHeight/showHeight;
			if(!(view instanceof ZoomPanView)){
				origin=new Point(unscale(paneWidth/2),unscale(paneHeight/2)
						).shifted(plotShift);
			}
			else{
				ZoomPanView zoom=(ZoomPanView)view;
				double planeW=zoom.planeWidth(),planeH=zoom.planeHeight();
				origin=new Point(-zoom.originToLeft(),zoom.originToTop());
				Point planeFocus=plot(zoom.focusAt());
				double viewerLeft=scale(planeFocus.x())-paneWidth/2,
					viewerTop=scale(planeFocus.y())-paneHeight/2;
				((PlaneHost)host).adjustToPlane((int)scale(planeW),(int)scale(planeH),
						(int)viewerLeft,(int)viewerTop);
			}
		}
		if(resize==RESHAPE){
			double setWidth=unscale(paneWidth),setHeight=unscale(paneHeight);
			Vector setShift=false?plotShift:
				new Vector(plotShift.x-(setWidth-showWidth)/2,
					plotShift.y-(setHeight-showHeight)/2*view.ySign());
			origin=new Point(unscale(((double)paneWidth)/2),unscale(((double)paneHeight)/2)
				).shifted(setShift);
			if(false)Util.println("PlaneCanvas.adjustToPaneSize: paneHeight="+paneHeight+
					" setHeight="+Doubles.sf(setHeight));
			if(false)Debug.printStackTrace(5);
			if(scale==scale&&(this.paneWidth!=paneWidth||this.paneHeight!=paneHeight)){
				view.setShowValues(setWidth,setHeight,setShift,scale);
				stamp=view.stamp();
				scale=view.scale();
			}
		  ((KViewer)facet.base()).refresh(Impact.CONTENT);
		}
		this.paneWidth=paneWidth;
		this.paneHeight=paneHeight;
	}
	private PlaneView planeView(){return (PlaneView)viewer().view();}
	Point plot(Point point){
		double x=scale(origin.x()+point.x()),y=scale(origin.y()+point.y()
				*planeView().ySign());
		return new Point(x,y);
	}
	double scale(double value){return value*scale;}
	Point unplot(double plotX,double plotY){
		double x=unscale(plotX)-origin.x(),y=(unscale(plotY)-origin.y())
				*planeView().ySign();
		return new Point(x,y);
	}
	double unscale(double scaled){
		if(scale!=scale)throw new IllegalStateException("No scale in "+Debug.info(this));
		return scaled/scale;
	}
	Vector checkSnap(Point check,double snapGap){
		Vector snap=null;
		for(int i=0;i<avatarSelectables.length&&snap==null;i++)
			snap=avatarSelectables[i].checkSnap(this,check,snapGap);
		return snap;
	}
	void panAreaSet(int left,int top){
		initialPortAtZeros&=left<=0&&top<=0;
		if(initialPortAtZeros)return;
		PlaneView plane=planeView();
		if(plane instanceof ZoomPanView)
			((ZoomPanView)plane).setFocusAt(
					unplot(left+paneWidth/2,top+paneHeight/2));
	}
	protected boolean consumeMouseEvent(int type,double atX,double atY,int mods){
		if(scale!=scale)return false;
		Point planeAt=unplot(atX,atY);
		return super.consumeMouseEvent(type,planeAt.x(),planeAt.y(),mods);
	}
	protected double hitGap(){return unscale(super.hitGap());}
	protected double markSize(){return unscale(super.markSize());}
	double markLeap(){return unscale(planeView().markLeapPixels());}
	public String toString(){
		return //Debug.info(this)+
		Util.fx(scale)+" "+new Point(paneWidth,paneHeight)+" "+origin;
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
