package facets.facet.toolkit.avatar;
import static facets.facet.FacetFactory.*;
import facets.core.app.StatefulViewable;
import facets.core.app.avatar.AvatarView;
import facets.core.app.avatar.Painter;
import facets.core.app.avatar.PainterSource;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.SAreaTarget;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.toolkit.avatar.AvatarCanvas.CanvasHost;
import facets.facet.toolkit.swing.ClipperSwing;
import facets.facet.toolkit.swing.ViewerMaster;
import facets.util.Debug;
import facets.util.Stateful;
import facets.util.shade.Shade;
import java.awt.AWTEvent;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.IOException;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.Timer;
import javax.swing.TransferHandler;
abstract class SwingAvatarMaster extends ViewerMaster implements CanvasHost{
	final class CanvasPane extends JPanel{
	  private Timer notifier;
		private final ViewerMaster viewer;
		private Dimension sizeThen;
		MouseEvent lastEvent;
		private int lastX,lastY,lastMods;
		private boolean waitCursorSet;
		protected void processMouseEvent(MouseEvent e){
			super.processMouseEvent(e);
			lastEvent=e;
			int id=e.getID();
			switch(id) {
			case MouseEvent.MOUSE_PRESSED:
				if(notifyInterim)notifier=new Timer(dragNotifyPause,
			  		new ActionListener(){		
					public void actionPerformed(ActionEvent e){
						canvas.consumeMouseEvent(AvatarCanvas.MOUSE_DRAG_NOTIFY,
								lastX,lastY,lastMods);
					}
				}){
			  	public boolean isRepeats(){return false;}
				};
				requestFocus();
				break;
			case MouseEvent.MOUSE_DRAGGED:
				dragging=true;
				if(notifier==null)break;
				if(dragNotifyWait)notifier.stop();
				if(notifyInterim)notifier.start();
				break;
			case MouseEvent.MOUSE_RELEASED:
				dragging=false;
				if(notifier==null)break;
				notifier.stop();
				break;				
			}
			if(!canvas.consumeMouseEvent(id,lastX=e.getX(),lastY=e.getY(),
					lastMods=true?0:e.getModifiers()<<13))
				SwingAvatarMaster.this.unconsumedMouseEvent(e);
		}
		CanvasPane(){
			viewer=SwingAvatarMaster.this;
  		enableEvents(AWTEvent.MOUSE_EVENT_MASK|AWTEvent.MOUSE_MOTION_EVENT_MASK|
  		    AWTEvent.KEY_EVENT_MASK);
  		setFocusTraversalKeysEnabled(false);
  		setTransferHandler(dndHandler);
  		if(false)setOpaque(false);
		}
		public void doLayout(){
		  super.doLayout();
		  Dimension sizeNow=getSize();
		  if(sizeNow.equals(sizeThen))return;
		  if(sizeNow.width<0||sizeNow.height<0)sizeNow=new Dimension();
		  sizeThen=sizeNow;
		  canvas.paneSizeSet(sizeNow.width,sizeNow.height);
	  }
		public void paintComponent(final Graphics g){
			double scale=((PlaneCanvas)canvas).scale;
			if(scale!=scale)return;
			ViewerTarget target=viewerTarget();
			AvatarView view=(AvatarView)target.view();
			Shade background=(Shade)view.backgroundStyle();
			setBackground(new Color(background.rgb()));
		  super.paintComponent(g);
		  painters.doPainting((Graphics2D)g.create());
			SAreaTarget areaRoot=target.areaParent().areaParent();
			if(target.isActive()
					&&areaRoot!=null&&areaRoot.indexableTargets().length>1){
				Graphics hilite=g.create();
				Dimension size=getSize();
				hilite.setColor(Color.lightGray);
				for(int xy=0,inset=1;xy<1;xy++,inset+=2)
					hilite.drawRect(xy,xy,size.width-inset,size.height-inset);
			}
		}
		protected void processMouseMotionEvent(MouseEvent e){
			processMouseEvent(e);
		}
	}
  final protected AvatarCanvas canvas;
	private final SwingCanvasPainters painters;
	private PainterSource painterSource;
	private boolean dragging;
	public SwingAvatarMaster(AvatarCanvas canvas){
    (this.canvas=canvas).host=this;
		painters=new SwingCanvasPainters(this);
  }	
	final public void setCanvasCursor(int cursor){
		findCanvasPane().setCursor(Cursor.getPredefinedCursor(cursor));
	}
	public boolean isScrollable(){
		return false;
	}
	private String areaTitle(){
		return viewerTarget().areaParent().title();
	}
	final public void setAndPaintPainters(Painter background,Painter[]viewPainters,
			Painter[]motionPainters){
		painters.setPainters(background,viewPainters,motionPainters);
		avatarPane().repaint();
	}
	public void refreshAvatars(Impact impact){
		if(!dragging)canvas.refreshViewPainters();
  }
  protected CanvasPane findCanvasPane(){
  	return(CanvasPane)avatarPane();
  }
	protected JComponent newAvatarPane(){
		return new CanvasPane();
	}
  protected void unconsumedMouseEvent(MouseEvent e){}
	public void launchDnD(){
		dndHandler.exportAsDrag(avatarPane(), findCanvasPane().lastEvent,
				TransferHandler.MOVE);
	}
	private final static TransferHandler dndHandler=new TransferHandler(){
		private SwingAvatarMaster from,to;
		private DataFlavor toFlavor;
		public int getSourceActions(JComponent c){
			from=getComponentViewer(c);
			if(from==null) throw new IllegalStateException("Null from in "+Debug.info(c));
			return COPY_OR_MOVE;
		}
		protected Transferable createTransferable(JComponent c){
			if(getComponentViewer(c)!=from) throw new IllegalArgumentException("Bad component "+Debug.info(c));
			return new ClipperSwing((StatefulViewable)from.viewerTarget().viewable,false).newClip();
		}
		public boolean canImport(JComponent c,DataFlavor[]flavors){
			to=getComponentViewer(c);
			if(to==null||from==null||
					to.viewerTarget().viewable==
						from.
						viewerTarget().
						viewable)return false;
			toFlavor=new ClipperSwing((StatefulViewable)to.viewerTarget().viewable,false).dataFlavor;
			for(int i=0;i<flavors.length;i++)
				if(flavors[i].equals(toFlavor)) return true;
			return false;
		}
		public boolean importData(JComponent c,Transferable t){
			if(!canImport(c,t.getTransferDataFlavors())) return false;
			try{
				StatefulViewable viewable=(StatefulViewable)to.viewerTarget().viewable;
				viewable.insertStatefuls((Stateful[])t.getTransferData(toFlavor));
				viewable.updateAfterEditAction();
				return true;
			}catch(UnsupportedFlavorException ufe){
				throw new RuntimeException(ufe);
			}catch(IOException ioe){
				throw new RuntimeException(ioe);
			}
		}
		protected void exportDone(JComponent c,Transferable data,int action){
			if(getComponentViewer(c)!=from) throw new IllegalArgumentException(
					"Bad component "+Debug.info(c));
			StatefulViewable viewable=(StatefulViewable)from.viewerTarget().viewable;
			if(action==MOVE){
				viewable.deleteSelection();
				viewable.updateAfterEditAction();
			}
			if(to!=null)to.viewerTarget().ensureActive(true);
			from.canvas.refreshViewPainters();
			to.canvas.refreshViewPainters();
			from=to=null;
		}
		private SwingAvatarMaster getComponentViewer(JComponent c){
			return !(c instanceof CanvasPane)?null
					:(SwingAvatarMaster)((CanvasPane)c).viewer;
		}
	};
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

