package facets.facet.toolkit.avatar;
import facets.core.app.avatar.PainterMaster.Outlined;
import facets.core.app.avatar.PainterSource.Transform;
import facets.util.Debug;
import facets.util.Identified;
import facets.util.Tracer;
import facets.util.Util;
import facets.util.geom.Point;
import facets.util.shade.Shade;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Arrays;
final class SwingPainterOutlined extends Tracer implements PickPainter,Identified{
	private static int identities;
	private final int identity=identities++;
	private final Outlined master;
	private final PlaneCanvas canvas;
	private Shape penShape,fillShape;
	private AffineTransform transform=new AffineTransform();
	private BasicStroke stroke;
	private int hashCode=Integer.MAX_VALUE;
	SwingPainterOutlined(Outlined master,PlaneCanvas canvas){
		this.master=master;
		this.canvas=canvas;
	}
	public int hashCode(){
		return hashCode==Integer.MAX_VALUE?
				hashCode=Arrays.hashCode(new Object[]{master,transform}):hashCode;
	}
	private void checkPaintShapes() {
		if(penShape!=fillShape)return;
		Shape shape=(Shape)master.getOutline();
		if(shape==null)throw new IllegalStateException("No shape in "+Debug.info(this));
		shape=transform==null?shape:transform.createTransformedShape(shape);
		Shade fill=master.getFill(),pen=master.getPen();
		if(fill!=null)fillShape=shape;
		if(pen==null)return;
		Object penStyle=master.penStyle();
		stroke=penStyle==Outlined.PEN_STYLE_USE_DEFAULT?
				new BasicStroke((float)canvas.unscale(1))
			:(BasicStroke)penStyle;
		penShape=false?stroke.createStrokedShape(shape):shape;
	}
	public void paintInGraphics(Object graphics){
		checkPaintShapes();
		Graphics2D g=(Graphics2D)graphics;
		Shade pen=master.getPen(),fill=master.getFill();
		if(fill!=null){
			g.setColor(new Color(fill.rgb()));
			g.fill(fillShape);
		}
		if(pen!=null){
			g.setColor(new Color(pen.rgb()));
			g.setStroke(stroke);
			g.draw(penShape);
		}
	}
	public Object checkCanvasHit(Point canvasAt,double hitGap){
		if(!master.isPickable())return null;
		double 
			canvasGap=canvas.unscale(hitGap),boxSize=canvasGap*2,
			boxX=canvasAt.x()-boxSize/2,
			ySign=-1,
			boxY=canvasAt.y()+boxSize/2*ySign;
		Rectangle2D hitBox=new Rectangle2D.Double(boxX,boxY,boxSize,boxSize);
		checkPaintShapes();
		if(!(penShape!=null&&penShape.intersects(hitBox)||
				fillShape!=null&&fillShape.intersects(hitBox))
				)return null;
		if(false)Util.println("SwingPainterOutlined.checkCanvasPick: ",
				fillShape.getBounds()
				+"\n"+
				hitBox.getBounds()
				);
		Point2D shapeAt=null;
		try{
			shapeAt=transform.inverseTransform(new Point2D.Double(
						canvasAt.x(),
					canvasAt.y()),null);
		}catch(NoninvertibleTransformException e){
			throw new RuntimeException(e.getMessage());
		}
		return new Pick(this,new Point(shapeAt.getX(),shapeAt.getY()),null);
	}
	void addTransform(Transform transform,boolean concatenate){
		if(concatenate)
			this.transform.concatenate((AffineTransform)transform);
		else this.transform=((AffineTransform)transform);
		penShape=fillShape=null;
	}
	public Object identity(){
		return identity;
	}
	public final void setTitle(String title){
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
