package facets.facet.toolkit.swing;
import facets.facet.toolkit.swing.AppDesktop.ContentPanel;
import facets.facet.toolkit.swing.AppDesktop.Sizes;
import facets.util.Debug;
import facets.util.Util;
import java.awt.Rectangle;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyVetoException;
import java.beans.VetoableChangeListener;
import javax.swing.JComponent;
import javax.swing.JInternalFrame;
import javax.swing.SwingUtilities;
import javax.swing.JInternalFrame.JDesktopIcon;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.InternalFrameListener;
final class AppDesktopFrame extends JInternalFrame{
	private static final boolean monitoring=false;
	public String toString(){
		return getTitle()+" selected="+isSelected()+" maximum="+isMaximum
		+" size="+getSize().width+" icon="+isIcon;
	}
	private final VetoableChangeListener changes=new VetoableChangeListener(){
		public void vetoableChange(PropertyChangeEvent e) 
				throws PropertyVetoException{
			String name=e.getPropertyName();
			if(name.equals("closed")){
				if(monitoring)Util.println("changes: ",AppDesktopFrame.this);
				if(!content.windowCanClose())throw new PropertyVetoException("",e);
				return;
			}
		}
	};
	private final InternalFrameListener states=new InternalFrameListener(){
		public void internalFrameActivated(InternalFrameEvent e){
			if(monitoring)Util.println("Activated: ",AppDesktopFrame.this);
			if(!desktop.settingActive)content.windowActivated();
		}
		public void internalFrameIconified(InternalFrameEvent e){
			if(monitoring)Util.println("Iconified: ",AppDesktopFrame.this);
			if(desktop.sizes.iconGap>0)return;
			Sizes nowSizes=new Sizes(desktop.sizes,desktopIcon.getSize().height);
			if(monitoring)Util.println("Iconified: nowSizes=",nowSizes+"sizes="+desktop.sizes);
			desktop.adjustToSizes(nowSizes);
			desktop.sizes=nowSizes;
		}
		public void internalFrameDeiconified(InternalFrameEvent e){
			if(monitoring)Util.println("Deiconified: ",AppDesktopFrame.this);
		}
		public void internalFrameDeactivated(InternalFrameEvent e){
			if(monitoring)Util.println("Deactivated: ",AppDesktopFrame.this);
		}
		public void internalFrameOpened(InternalFrameEvent e){
			if(monitoring)Util.println("Opened: ",AppDesktopFrame.this);
		}
		public void internalFrameClosing(InternalFrameEvent e){
			if(monitoring)Util.println("Closing: ",AppDesktopFrame.this);
		}
		public void internalFrameClosed(InternalFrameEvent e){
			if(monitoring)Util.println("Closed: ",AppDesktopFrame.this);
		}
	};
	private final PropertyChangeListener maximising=new PropertyChangeListener(){
		public void propertyChange(PropertyChangeEvent e){
			if(!e.getPropertyName().equals("maximum"))return;
			if(false)Util.println("Maximising: ",AppDesktopFrame.this);
			boolean maximum=isMaximum();
			content.windowMaximumSet(desktop.framesMaximum=maximum);
			if(!maximum)setBounds(restoreBounds);
		}
	};
	final ContentPanel content;
	private final AppDesktop desktop;
	private Rectangle restoreBounds;
	AppDesktopFrame(AppDesktop desktop,ContentPanel content){
		this.content=content;
		this.desktop=desktop;
		setMaximizable(true);
		setResizable(true);
		setClosable(true);
		setIconifiable(true);
		setVisible(true);
		updateTitle();
		getContentPane().add(content);
		pack();		
		addInternalFrameListener(states);
		addPropertyChangeListener(maximising);
		addVetoableChangeListener(changes);
	}
	void trySetMaximum(boolean onOff){
		try{
			setMaximum(onOff);
			if(monitoring)Util.println("Frame.trySetMaximum: ",onOff+" "+this);
		}catch(PropertyVetoException e){
			throw new RuntimeException(e);
		}
	}
	void trySetSelected(boolean on){
		try{
			if(monitoring)Util.println("Frame.trySetSelected: "+on+" ",this);
			setSelected(on);
			if(monitoring)Util.println("Frame.~trySetSelected: "+on+" ",this);
		}catch(PropertyVetoException e){
			throw new RuntimeException(e);
		}
	}
	void setRestoreBounds(Rectangle bounds){
		boolean maximum=isMaximum();
		removePropertyChangeListener(maximising);
		try{
			boolean toggleMaximum=false;
			if(toggleMaximum)setMaximum(false);
			setBounds(restoreBounds=bounds);
			if(false)revalidate();
			if(toggleMaximum)setMaximum(maximum);
		}catch(PropertyVetoException e){
			throw new RuntimeException(e);
		}
		addPropertyChangeListener(maximising);
		if(monitoring)Util.println("Frame.setRestoreBounds: size=",bounds.height+" "+this);
	}
	void updateTitle(){
		setFrameIcon(content.windowIcon());
		setTitle(content.windowTitle());
	}
	boolean framesContent(ContentPanel panel){
		return getContentPane().getComponent(0)==panel;
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
