package facets.facet.toolkit.swing;
import static facets.core.app.AppConstants.*;
import static facets.util.app.AppValues.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.Dialogs;
import facets.facet.app.FacetDialogSurface;
import facets.util.FileSpecifier;
import facets.util.Util;
import facets.util.app.AppFileValues;
import facets.util.app.AppValues;
import facets.util.datatree.Nodes;
import facets.util.datatree.TypedNode;
import facets.util.datatree.ValueNode;
import java.awt.Component;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.util.Arrays;
import java.util.Collections;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.filechooser.FileFilter;
final class FileChooser extends JFileChooser{
  private final AppFileValues fileValues;
	private final ActionAppSurface app;
	protected FileSpecifier activeSpecifier;
	private JDialog dialog;
	private boolean moved;
	private int returnValue;
	FileChooser(ActionAppSurface app){
		this.app=app;
		fileValues=new AppFileValues(app.values){};
	}
	int displayDialog(Component parent){
		boolean parentVisible=parent.isVisible();
		String boundsKey=Dialogs.KEYTOP_BOUNDS+Util.shortTypeNameKey(this);
		ValueNode boundsValues=app.values.state(PATH_CORE,STATE_NODE_DIALOGS);
		int[]intsThen=boundsValues.getInts(boundsKey);
		Rectangle boundsThen=intsThen.length<4?null
				:new Rectangle(intsThen[0],intsThen[1],intsThen[2],intsThen[3]);
		if(true||dialog==null){
			dialog=createDialog(parentVisible?parent:null);
			dialog.addWindowListener(new WindowAdapter(){
				public void windowClosing(WindowEvent e){
					returnValue=CANCEL_OPTION;
				}
			});
			returnValue=ERROR_OPTION;
			rescanCurrentDirectory();
		}
		Point at=!moved?boundsThen!=null?boundsThen.getLocation()
				:FacetDialogSurface.calculateSmartDialogAt(dialog.getSize(),app.host())
			:dialog.getLocation();
		if(parentVisible)dialog.setLocation(at);
		dialog.setVisible(true);
		moved|=!dialog.getLocation().equals(at);
		Rectangle bounds=dialog.getBounds();
		int[]intsNow={bounds.x,bounds.y,bounds.width,bounds.height};
		if(parentVisible)boundsValues.put(boundsKey,intsNow);
		return returnValue;
	}
	public void approveSelection(){
		returnValue=APPROVE_OPTION;
		dialog.setVisible(false);
	}
  public void cancelSelection(){
		returnValue=CANCEL_OPTION;
		dialog.setVisible(false);
	}
	File getFile(File proposed,FileSpecifier[]specifiers){
		boolean saving=proposed!=null;
		setDialogType(saving?SAVE_DIALOG:OPEN_DIALOG);
		if(!saving)setDialogTitle(app.title()+" - Open File");
		setCurrentDirectory(fileValues.stateGetPath());
		if(saving)setSelectedFile(proposed);
		resetChoosableFileFilters();
		if(specifiers.length>0)setAcceptAllFileFilterUsed(false);
		for(final FileSpecifier specifier:specifiers)
			setFileFilter(new FileFilter(){
				@Override
				public String getDescription(){
					return specifier.rubric;
				}
				@Override
				public boolean accept(File path){
					FileSpecifier active=activeSpecifier=specifier;
					return path.isDirectory()||active==FileSpecifier.ALL
						||active.specifies(path);
				}
			});
		rescanCurrentDirectory();
		int returnValue=ERROR_OPTION;
		File selected;
		while(true){
			returnValue=displayDialog((Component)app.host().wrapped());
			selected=getSelectedFile();
			boolean canStop=saving||returnValue==CANCEL_OPTION;
			canStop|=selected==null||(selected!=null&&selected.exists());
			if(canStop)break;
		}
		fileValues.statePutPath(getCurrentDirectory());
		return returnValue!=APPROVE_OPTION?null
				:saving?activeSpecifier.specifiedFile(selected)
				:selected;
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
