package facets.facet.toolkit.swing;
import static facets.core.superficial.app.SViewer.*;
import static javax.swing.SwingUtilities.*;
import static javax.swing.text.StyleConstants.*;
import facets.core.app.HtmlContent;
import facets.core.app.HtmlView;
import facets.core.app.HtmlContent.HtmlSelected;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.ViewerTarget;
import facets.util.Debug;
import facets.util.StringFlags;
import facets.util.Times;
import facets.util.app.ProvidingCache.ItemProvider;
import java.awt.AWTEvent;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.StringReader;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.DefaultCaret;
import javax.swing.text.Document;
import javax.swing.text.EditorKit;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyledDocument;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.View;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.StyleSheet;
/**
{@link ViewerMaster} wrapping a {@link JEditorPane}. 
<p>{@link HtmlPaneMaster} is in some ways the most complex {@link ViewerMaster}:
<ul>
<li>It can display its own HTML source code by switching {@link EditorKit}s to match 
{@link HtmlView#showSource()}.
<li>In either display style line wrapping matches {@link HtmlView#wrapLines()}.  
<li>Content can be passed as as an {@link HtmlContent}, or as 
a {@link String} or {@link String}[] for setting as the source of an internal
 {@link HtmlContent}. 
<li>Where content is passed as a {@link String}[] and  
{@link HtmlView#quickLineHeight()} returns a useful value, it truncates 
the source of the internal {@link HtmlContent} to match the current pane size.
<li>Content renderings are cached as complete {@link JEditorPane}s, keyed against the  
{@link HtmlContent#source}.   
</ul>
<p>These features require the following content stack:
<ol>
<li>An {@link HtmlContent} wrapping HTML source which may have been passed as content, 
or created from a truncated {@link String}[] passed as content. 
<li>A {@link StyledDocument} created from {@link HtmlContent#getKitDocument(EditorKit)} for 
each change in HTML source. 
<li>A {@link JEditorPane} for each new {@link StyledDocument} to enable caching of 
its {@link View} tree. 
<li>A {@link HtmlWrapPane} to manage text wrap in the current {@link JEditorPane} and with a 
substitute {@link MouseListener} and {@link MouseWheelListener} to intercept attempts to 
adjust a {@link JEditorPane} displaying truncated content.    
</ol>
 */
final class HtmlPaneMaster extends ViewerMaster{
	private static final Dimension screen=Toolkit.getDefaultToolkit().getScreenSize();
	private final static boolean timing=true;
	final static EditorKit sourceKit=new StyledEditorKit(),
		renderKit=new HTMLEditorKit(){
			StyleSheet styles=new StyleSheet();
			public StyleSheet getStyleSheet(){
				return true?styles:super.getStyleSheet();
			};
		};
	private void clearQuickParas(){
		if(quickPaneHeight<=0)return;
		JComponent pane=avatarPane();
		int wheelListeners=pane.getMouseWheelListeners().length;
		if(wheelListeners==0)return;
		if(false)trace(".clearQuickParas: newEditor=",newEditor);
		if(true||newEditor)pane.setCursor(Cursor.getPredefinedCursor(CURSOR_WAIT));
		viewerTarget().ensureActive(true);
	}
	private final HtmlContent htmlInternal=new HtmlContent("htmlInternal",
		"Your source goes here");
	private final KitSwing kit;
	private final StringFlags hints;
	private boolean changingSelection,newEditor;
	private int quickPaneHeight;
	private Document doc;
	HtmlPaneMaster(KitSwing kit,StringFlags hints){
		this.kit=kit;
		this.hints=hints;
	}
	protected JComponent newAvatarPane(){
		HtmlWrapPane pane=new HtmlWrapPane(newEditor()){
			void _refreshForEditor(JEditorPane e){
				super.refreshForEditor(e);
				e.setCursor(getCursor());
				for(MouseListener l:e.getMouseListeners())e.removeMouseListener(l);
				for(MouseListener l:getMouseListeners())e.addMouseListener(l);
				for(MouseWheelListener l:e.getMouseWheelListeners())e.removeMouseWheelListener(l);
				for(MouseWheelListener l:getMouseWheelListeners())e.addMouseWheelListener(l);
			}
		};
		pane.addMouseWheelListener(new MouseWheelListener(){
			public void mouseWheelMoved(MouseWheelEvent e){
				clearQuickParas();
			}
		});
		pane.addMouseListener(new MouseAdapter(){
			public void mouseReleased(MouseEvent e){
				clearQuickParas();
			}
			public void mousePressed(MouseEvent e){
				clearQuickParas();
			}
			public void mouseClicked(MouseEvent e){
				clearQuickParas();
			}
		});
		return pane;
	}
	public void refreshAvatars(Impact impact){
		if(changingSelection)return;
		ViewerTarget target=viewerTarget();
		HtmlView view=(HtmlView)target.view();
		HtmlWrapPane wrapPane=(HtmlWrapPane)avatarPane();
		JViewport viewport=wrapPane.findViewport();
		final JComponent editorThen=wrapPane.editor();
		if(false&&inInactivePane(editorThen))return;
		boolean wrapLines=view.wrapLines();
		wrapPane.refreshForWrap(wrapLines);
		if(timing)traceDebug(": target="+Debug.info(target)+" editorThen=",editorThen);
		final Object content=target.selection().content();
		final HtmlContent html=content instanceof HtmlContent?
				(HtmlContent)content:htmlInternal;
		boolean quickParas=false;
		if(html==htmlInternal){
			if(content instanceof String)html.setState(content);
			else{
				String[]paraLines=(String[])content;
				int portHeight=viewport.getSize().height,
					quickParaHeight=(int)(view.quickLineHeight()*kit.layoutFactor());
				quickParas=!wrapLines&&quickParaHeight>=0&&portHeight>0&&!target.isActive();
				quickPaneHeight=!quickParas?-1:paraLines.length*quickParaHeight;
				int stopLineAt=!quickParas?paraLines.length:(screen.height/1/quickParaHeight),
					lineAt=0;
				boolean debug=false;
				if(debug)trace(".refreshAvatars: quickParaHeight="+quickParaHeight+
						" lines="+(paraLines.length-2)+" stopLineAt="+stopLineAt);
				StringBuilder setSource=new StringBuilder();
				for(String line:paraLines)if(lineAt++<stopLineAt){
						setSource.append(line.trim());
						setSource.append("\n<p>");
					}
					else break;
				setSource.append(paraLines[paraLines.length-1]);
				if(debug)trace(".refreshAvatars: quickPaneHeight="+quickPaneHeight+
						" setSource=",setSource.length());
				html.setState(setSource.toString());
			}
		}
		final boolean showSource=showSource();
		if(timing)trace(": getting editor source=",html.source().length());
		JEditorPane editorNow=new ItemProvider<JEditorPane>(base().providingCache(),this,
				HtmlPaneMaster.class.getSimpleName()+".refreshAvatars"){
			private final boolean singleEditor=false;
			@Override
			protected boolean passThrough(){
				return singleEditor;
			}
			@Override
			public CancelStyle cancelStyle(){
				return CancelStyle.Timeout;
			}
			@Override
			protected long advanceByteCount(){
				return html.source().length();
			};
			@Override
			protected JEditorPane newItem(){
				if(timing)trace(": setting html=",html.source().length());
				JEditorPane editor=singleEditor?(JEditorPane)editorThen:newEditor();
				EditorKit kit=showSource?sourceKit:renderKit;
				Document doc=getKitDocument(kit);
				editor.setEditorKit(kit);
				editor.setDocument(doc);
				newEditor=true;
				return editor;
			}
		}.getForValues(this,html.source(),showSource,wrapLines);
		if(timing)traceDebug(": got editorNow=",editorNow);
		wrapPane.refreshForEditor(editorNow);
		editorNow.getCaret().setDot(-1);
		viewport.scrollRectToVisible(new Rectangle(0,0,0,0));
		((JScrollPane)viewport.getParent()).setWheelScrollingEnabled(!quickParas);
		newEditor=false;
		if(listener==null||!editorNow.isEditable())return;
		editorNow.getDocument().addDocumentListener(listener);
		HtmlSelected runs=(HtmlSelected)target.selection().selection();
	}
	private Document getKitDocument(EditorKit kit){
		boolean notSource=kit instanceof HTMLEditorKit;
		Document doc=false?new HtmlSelectingDocument():kit.createDefaultDocument();
		if(doc.getLength()==0)try{
			kit.read(new StringReader(htmlInternal.source()),doc,0);
		}catch(Exception e){
			throw new RuntimeException(e);
		}	
		if(notSource)return this.doc=doc;
		SimpleAttributeSet atts=new SimpleAttributeSet();
		atts.addAttribute(Family,"Courier New");
		((StyledDocument)doc).setCharacterAttributes(0,doc.getLength(),atts,true);
		return doc;
	}
	private boolean inInactivePane(final JComponent editor){
		AppDesktopFrame frame=(AppDesktopFrame)getAncestorOfClass(
				AppDesktopFrame.class,editor);
		boolean inInactivePane=frame!=null&&!frame.isSelected();
		return inInactivePane;
	}
	@Override
	protected void traceOutput(String msg){
		if(true||timing)Times.printElapsed(Debug.info(this)+msg);
		else if(true)super.traceOutput(msg);
	}
	private JEditorPane newEditor(){
		JEditorPane editor=new JEditorPane(){
			public void paintComponent(Graphics g){
				if(false&&timing)trace(".paint size="+getSize());
				super.paintComponent(g);
				if(false&&timing)trace(".~paint");
				SwingUtilities.invokeLater(new Runnable(){public void run(){
					setCursor(Cursor.getPredefinedCursor(CURSOR_TEXT));
				}});
			}
			public Dimension getPreferredSize(){
				Dimension superSize=super.getPreferredSize();
				return new Dimension(superSize.width,
						quickPaneHeight<=0?superSize.height:quickPaneHeight);
			}
			public void removeMouseWheelListener(MouseWheelListener l){
				super.removeMouseWheelListener(l);
				disableEvents(AWTEvent.MOUSE_WHEEL_EVENT_MASK);
			};
		};
		boolean live=((HtmlView)viewerTarget().view()).isLive();
		editor.setEditable(live);
		editor.setDragEnabled(live);
		if(false)editor.setCaret(new DefaultCaret(){
			public void focusLost(FocusEvent e){}
		});
		return editor;
	}
	@Override
	protected int defaultCursor(){
		return CURSOR_TEXT;
	}
	private boolean showSource(){
		return((HtmlView)viewerTarget().view()).showSource();
	}
	private final DocumentListener listener=true?null:new DocumentListener(){
		public void changedUpdate(DocumentEvent e){
			traceDebug(".changedUpdate: ",HtmlPaneMaster.this);
		}
		public void removeUpdate(DocumentEvent e){
			traceDebug(".removeUpdate: ",HtmlPaneMaster.this);
		}
		public void insertUpdate(DocumentEvent e){
			viewerTarget().selectionEdited(null,e, false);
			traceDebug(".insertUpdate: ",HtmlPaneMaster.this);
		}
	};
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
