package facets.facet.toolkit.swing;
import facets.core.app.ArrayPath;
import facets.core.app.PathSelection;
import facets.core.app.TableView;
import facets.core.superficial.Notifying;
import facets.core.superficial.STarget;
import facets.core.superficial.SToggling;
import facets.core.superficial.Notifying.Impact;
import facets.core.superficial.app.ViewerTarget;
import facets.facet.FacetFactory;
import facets.util.Strings;
import facets.util.Util;
import facets.util.app.Events;
import facets.util.datatree.ValueNode;
import facets.util.datatree.ValueProxy;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.Graphics;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.text.Format;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;
final class TablePaneMaster extends ViewerMaster{
  private static final Font FONT_CELL=new Font("SanSerif",Font.PLAIN,12),
  	FONT_HEADER=new Font("SanSerif",Font.BOLD,12);
  private final TableModel model=new AbstractTableModel(){
    public Class getColumnClass(int col){
    	Class type=content.get(0).get(col).getClass();
			return view().getColumnFormat(col)!=null?String.class:type;
    }
    public Object getValueAt(int row,int col){
			Object value=content.get(row).get(col);
			Format format=view().getColumnFormat(col);
			return format!=null?format.format(value):value;
		}
		public int getColumnCount(){
    	ValueProxy firstRow=content.get(0); 
    	firstRow.get(0);
			return firstRow.valueCount();
    }
    public String getColumnName(int col){
    	return base().kit.getDecorationTitle(view().getColumnTitle(col));
    }
    public int getRowCount(){
    	return content.size();
    }
    public boolean isCellEditable(int row,int col){
    	return view().isColumnEditable(col);
    }
    public void setValueAt(Object value,int row,int col){
    	ValueProxy gotRow=content.get(row);
			gotRow.put(col,value);
    	Object[]array=content.toArray();
			viewerTarget().selectionEdited(PathSelection.newSingle(array,
    			new ArrayPath(array,contentAt)),new Object[]{gotRow},false);
		}
  };
	private final ListSelectionListener selections=new ListSelectionListener(){
	  public void valueChanged(ListSelectionEvent e){
	    if(e.getValueIsAdjusting())return;
	    ListSelectionModel list=(ListSelectionModel)e.getSource();
	    if(list.isSelectionEmpty())return;
	  	int tableAt=list.getMinSelectionIndex();
  		contentAt=sorter==null?tableAt:sorter.convertRowIndexToModel(tableAt);
		  if(tableAt!=thenTableAt){
		  	thenTableAt=tableAt;
		  	if(!view().allowSelectOnEdit()){
		  		Object[] array=content.toArray();
					viewerTarget().selectionChanged(PathSelection.newSingle(array,
		  				new ArrayPath(array,contentAt)));
				}
		  }
	  }
	};
	private TableRowSorter sorter;
  private List<ValueProxy>content;
	private int thenTableAt=-1,contentAt;
	private final ValueNode stateValues;
	private String widthKey;
	private final SToggling sorting=new SToggling("Sorting",false,
			new SToggling.Coupler(){
		public void stateSet(SToggling t){
			viewerTarget().notifyMonitor(Notifying.Impact.DEFAULT);
		}
	});
  TablePaneMaster(ValueNode stateValues,String widthKey){
		this.stateValues=stateValues;
		this.widthKey=widthKey;
	}
	public void refreshAvatars(Impact impact){
	  JTable table=(JTable)avatarPane();
		ViewerTarget viewer=viewerTarget();
	  ValueProxy[]contentNow=(ValueProxy[])viewer.selection().content();
	  if(content.size()!=contentNow.length)
	  	sorter.modelStructureChanged();
		content=Arrays.asList(contentNow);
	  updateTable(table,viewer);
	  table.repaint();
	}
	protected JComponent newAvatarPane(){
  	final ViewerTarget viewer=viewerTarget();
	  content=Arrays.asList((ValueProxy[])viewer.selection().content());
		TableView view=view();
    final JTable table=new JTable(model){
    	{
	    setFont(FONT_CELL);
	    setAutoResizeMode(AUTO_RESIZE_SUBSEQUENT_COLUMNS);
	    setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    setFillsViewportHeight(true);
	    setRowSorter(sorter=new TableRowSorter(model){
	  		@Override
	  		public void toggleSortOrder(int column){
	  			super.toggleSortOrder(column);
	  	  	sorting.set(true);
	  		}
	  	});
	    if(false)setColumnSelectionAllowed(false);
	    if(false)setRowSelectionAllowed(true);
	    if(false)setDefaultRenderer(Date.class,new TableCellRenderer(){
	    	final TableCellRenderer date=(TableCellRenderer)defaultRenderersByColumnClass.get(
	    			Date.class);
	      public Component getTableCellRendererComponent(JTable table,Object value,
	          boolean selected,boolean hasFocus,int row,int col){
	      	JLabel render=(JLabel)date.getTableCellRendererComponent(table,value,
	      			selected,hasFocus,row,col);
	        return render;
	      }
	    });
	    JComponent header=getTableHeader();
	    header.setFont(FONT_HEADER);
	    header.setForeground(Color.DARK_GRAY);
      final TableColumnModel cols=getColumnModel();
      final int colCount=cols.getColumnCount();
      updateTable(this,viewer);
      int[]widths=stateValues.getInts(widthKey);
      for(int i=0;i<widths.length;i++)
  			cols.getColumn(i).setPreferredWidth(widths[i]);
      PropertyChangeListener colListener=new PropertyChangeListener(){
        int[]widths=new int[colCount];
        public void propertyChange(PropertyChangeEvent e){
        	boolean changed=false;
          for(int i=0;i<widths.length;i++){
          	int width=cols.getColumn(i).getWidth();
          	changed|=width!=widths[i];
						widths[i]=width;
          }
          if(!changed)return;
        	if(false)trace(".propertyChange: " ,widthKey+
          		" "+Strings.intsString(widths));  
					stateValues.put(widthKey,Strings.intsString(widths));
					if(sorter.getSortKeys().size()>0){
						sorter.modelStructureChanged();
		  	  	sorting.set(false);
					}
        }
      };    
      for(int i=0;i<colCount;i++)
				cols.getColumn(i).addPropertyChangeListener(colListener);
      getSelectionModel().addListSelectionListener(selections);
    }};
		return table;
  }
	private void updateTable(JTable table,ViewerTarget viewer){
		int[]offsets=((PathSelection)viewer.selection()).paths[0].offsets;
	  ListSelectionModel selections=table.getSelectionModel();
		int nowRow=selections.getMinSelectionIndex(),
	  	setRow=offsets.length==1?-1:sorter.convertRowIndexToView(offsets[1]);
		if(setRow<0)selections.clearSelection();
		else if(setRow!=nowRow)selections.setSelectionInterval(setRow,setRow);
	}
	@Override
	protected STarget[]targets(){
		return new STarget[]{sorting};
	}
	private TableView view(){
		return (TableView)viewerTarget().view();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

