package facets.facet.toolkit.swing;
import facets.util.Debug;
import facets.util.Util;

import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import javax.swing.ButtonModel;
import javax.swing.Icon;
import javax.swing.JCheckBox;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.plaf.ActionMapUIResource;
class TristateCheckBox extends JCheckBox {
	
	//Store enabled state
	private boolean enabled=true;

	//Listener on model changes to maintain correct focusability
	final private ChangeListener modelListener=new ChangeListener(){
		
		public void stateChanged(ChangeEvent e){

			//Get model state, do HK change if necessary
			boolean enabledNow=getModel().isEnabled();
			if(enabledNow!=enabled)
				TristateCheckBox.this.setFocusable(enabled=enabledNow);
		}
	};

	/**
	Convenience constructor setting no icon. 
	 @param text passed to core constructor
	 */
	public TristateCheckBox(String text) {
		this(text, null);
	}

	/**
	Core constructor. 
	 @param text passed to superclass 
	 @param icon passed to superclass
	 */
	public TristateCheckBox(String text, Icon icon) {
		super(text, icon);
		
		//Set default single model
		setModel(new TristateButtonModel(false));
		
		//Clever stuff from HK
		super.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				TristateCheckBox.this.iterateState();
			}
		});
		ActionMap actions = new ActionMapUIResource();
		actions.put("pressed", new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				TristateCheckBox.this.iterateState();
			}
		});
		actions.put("released", null);
		SwingUtilities.replaceUIActionMap(this, actions);
	}
	public TristateCheckBox(){
		this("TristateCheckBox");
	}
	//Next two methods implement new API by delegation to model
	public void setIndeterminate() {
		triModel().setIndeterminate();
	}
	public boolean isIndeterminate() {
		return triModel().isIndeterminate();
	}
	
	//Overrides superclass method
	public void setModel(ButtonModel newModel){
		super.setModel(newModel);
		
		//Listen for enable changes
		if(model instanceof TristateButtonModel)
			model.addChangeListener(modelListener);
	}

	//Empty override of superclass method
	public void addMouseListener(MouseListener l) {}

	//Mostly delegates to model 
	private void iterateState() {

		//Maybe do nothing at all?
		if(!enabled)return;
		
		//From HK
		grabFocus();
		
		//Iterate state and fire ActionEvent
		String command=triModel().iterateState();		
		fireActionPerformed(new ActionEvent(this, 0, command));
	}

	//Convenience cast
	private TristateButtonModel triModel() {
		return (TristateButtonModel) getModel();
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

