package facets.facet.toolkit.swing;
import static javax.swing.SwingUtilities.*;
import facets.core.app.ActionAppSurface;
import facets.core.app.AppConstants;
import facets.core.app.Dialogs;
import facets.core.app.SurfaceServices;
import facets.core.superficial.SFacet;
import facets.core.superficial.app.SSurface;
import facets.facet.app.FacetAppSurface;
import facets.facet.app.WindowAppHost;
import facets.facet.toolkit.Kit;
import facets.util.Debug;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;
final class WindowHostSwing extends WindowAppHost{
	private final JMenuBar menuBar=new JMenuBar(){
		protected boolean processKeyBinding(KeyStroke ks,KeyEvent e, int condition, 
				boolean pressed) {
			if(false)trace(".processKeyBinding: e=",e);
			return super.processKeyBinding(ks, e, condition, pressed);
		};
	};
	private final JFrame frame;
	private final KitSwing kit;
	private final HostWrap hostWrap;
	private Splash splash;
	WindowHostSwing(FacetAppSurface app,KitSwing kit){
		super(app);
		this.kit=kit;
		Kit.widgets+=2;
		frame=new JFrame();
		frame.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		frame.addWindowListener(new WindowAdapter(){
		  public void windowClosing(WindowEvent e){
		  	WindowHostSwing.this.requestAppClose();
			}
		});
		hostWrap=new HostWrap(kit,menuBar);
	}
  public void splashUp(String msg){
		(splash=new Splash(msg,kit.getDecorationIcon(AppConstants.NATURE_APP_ICON_LARGE, false))).up();
	}
  public void splashDown(){
		if(splash!=null)splash.down();
	}
  public void runHostedApp(){
		Runnable openApp=new Runnable(){public void run(){
			app.openApp();
		}};
		if(false)openApp.run();
		else SwingUtilities.invokeLater(openApp);
	}
	protected void openWindow(){
		super.openWindow();
		Runnable openFrame=new Runnable(){public void run(){
			frame.setJMenuBar(menuBar);
		  frame.getContentPane().add((Component)hostWrap.wrapped());
			ImageIcon icon=(ImageIcon)kit.getAppIcon(app);
			if(icon!=null)frame.setIconImage(icon.getImage());
			frame.setVisible(true);
		}};
		if(isEventDispatchThread())openFrame.run();
		else invokeLater(openFrame);
	}
	protected void requestAppClose(){
		if(!app.closeAcceptable(WindowHostSwing.this))return;
		frame.dispose();
		System.exit(0);
	}
	public Object wrapped(){
	  return frame;
	}
	public void setTitle(String text){
		frame.setTitle(kit.decodeTitleText(text));
	}
	protected Rectangle windowBounds(){
		Rectangle bounds=frame.getBounds();
		return frame.isVisible()?bounds:new Rectangle();
	}
	protected Dimension screenSize(){
		if(true)return frame.getToolkit().getScreenSize();
		Rectangle virtualBounds=new Rectangle();
		GraphicsEnvironment ge=GraphicsEnvironment.getLocalGraphicsEnvironment();
		GraphicsDevice[]gs=ge.getScreenDevices();
		for(int j=0;j<gs.length;j++){
			GraphicsDevice gd=gs[j];
			GraphicsConfiguration[]gc=gd.getConfigurations();
			for(int i=0;i<gc.length;i++){
				virtualBounds=virtualBounds.union(gc[i].getBounds());
			}
		}
		return virtualBounds.getSize();
	}
	protected void setWindowBounds(final Rectangle bounds){
		Runnable doRun=new Runnable(){public void run(){
			frame.setBounds(bounds);frame.doLayout();
		}};
		if(false)SwingUtilities.invokeLater(doRun);
		else doRun.run();
	}
	public FacetLayout newDesktopLayout(SFacet contentFacet,LayoutFeatures f){
		return hostWrap.newDesktopLayout(contentFacet,f);
	}
	public FacetLayout newAppletLayout(SFacet content,SFacet toolPanel,
			SFacet[]menus,SurfaceServices services, boolean panelToSide){
		throw new RuntimeException("Not implemented in "+Debug.info(this));
	}
	public void setLayout(FacetLayout layout){
		hostWrap.setLayout(layout);
		updateLayout(app);
	}
	public void updateLayout(SSurface surface){
		hostWrap.updateLayout(surface);		
	}
	public SurfaceServices activeServices(){
		return hostWrap.activeServices();
	}
	public final Dialogs newDialogs(ActionAppSurface app){
		return new DialogsSwing(app,kit);
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

