package facets.facet.toolkit.swing.tree;
import facets.core.app.NodePath;
import facets.util.Debug;
import facets.util.OffsetPath;
import facets.util.Titled;
import facets.util.Util;
import facets.util.app.ProvidingCache;
import facets.util.app.ProvidingCache.ItemProvider;
import facets.util.datatree.Nodes;
import facets.util.datatree.TypedNode;
import java.awt.Insets;
import java.util.Collections;
import java.util.Enumeration;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;
final class MasterPathTree extends OffsetPathTree{
	private final boolean inGraphWrapper,debug=false;
	private final ProvidingCache pathStore=new ProvidingCache(20,null){
		protected boolean trace(){
			return false;
		};
	};
	private String lastKey=newExpansionsKey(TypedNode.NULL_NODE);
	public String convertValueToText(Object value,boolean selected,
			boolean expanded,boolean leaf,int row,boolean hasFocus){
		return super.convertValueToText(value instanceof Titled?((Titled)value).title():value,
				selected,expanded,leaf,row,hasFocus);
	}
	MasterPathTree(TreeModel model,boolean inGraphWrapper){
		super(model);
		this.inGraphWrapper=inGraphWrapper;
		setShowsRootHandles(true);
		getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
	}
	public OffsetPath newOffsetPath(TreePath eventPath){
		Object last=eventPath.getLastPathComponent();
		TreePath nodePath=last instanceof TypedNode?eventPath:eventPath.getParentPath();
		TypedNode lastNode=(TypedNode)nodePath.getLastPathComponent(),
			ancestry[]=Nodes.ancestry(lastNode);
		NodePath ancestryPath=new NodePath(ancestry);
		if(debug)Util.println("PathTree.newValueAtNodePath: nodePath==fullPath=",
				nodePath==eventPath);
		if(nodePath==eventPath)return ancestryPath;
		int rowAt=getRowForPath(nodePath)+1,valueAt=getRowForPath(eventPath),pathAt=-1;
		for(;rowAt<=valueAt;rowAt++){
			TreePath rowPath=getPathForRow(rowAt);
			boolean underNodePath=nodePath.equals(rowPath.getParentPath()),
				isValue=rowPath.getLastPathComponent()instanceof StringBuilder;
			if(debug)Util.println("PathTree.newValueAtNodePath: underNodePath="+underNodePath+
					" isValue="+isValue);
			if(underNodePath&&isValue)pathAt++;
		}
		if(debug)Util.println("PathTree.~newValueAtNodePath: pathAt="+pathAt
				+" valueAt="+valueAt+" rowAt="+rowAt);
		return ancestryPath.valueAtChecked(pathAt);
	}
	public void setOffsetPath(OffsetPath offsets){
		TreePath nodePath=new TreePath(offsets.members(getModel().getRoot()));
		int pathValueAt=((NodePath)offsets).valueAt();
		if(pathValueAt>=0){
			expandPath(nodePath);
			int rowAt=getRowForPath(nodePath)+1,valueAt=-1;
			for(;valueAt<pathValueAt;rowAt++)try{
				TreePath rowPath=getPathForRow(rowAt);
				boolean underNodePath=nodePath.equals(rowPath.getParentPath()),
					isValue=rowPath.getLastPathComponent()instanceof StringBuilder;
				if(debug)Util.println("PathTree.updateToSelectionPath: underNodePath="+
						underNodePath+" isValue="+isValue);
				if(underNodePath&&isValue)valueAt++;
			}
			catch(Exception e){
				return;
			}
			if(debug)Util.println("PathTree.~updateToSelectionPath: pathAt="+pathValueAt
					+" valueAt="+valueAt+" rowAt="+rowAt);
			nodePath=getPathForRow(rowAt-1);
		}
		setSelectionPath(nodePath);
		scrollPathToVisible(nodePath);
	}
	void storeExpansionPaths(){
		TypedNode root=(TypedNode)getModel().getRoot();
		String key=newExpansionsKey(root);
		Enumeration expanded=getExpandedDescendants(new TreePath(root));
		final TreePath[]paths=expanded==null?new TreePath[]{}:
	    (TreePath[])Collections.list(expanded).toArray(new TreePath[]{});
		newOffsetsProvider(paths).getNewForValues(lastKey=key);
	}
	private ItemProvider<OffsetPath[]>newOffsetsProvider(final TreePath[]paths){
		return new ItemProvider<OffsetPath[]>(pathStore,this,MasterPathTree.class.getSimpleName()){
			@Override
			public CancelStyle cancelStyle(){return CancelStyle.None;}
			protected OffsetPath[]newItem(){
				if(paths==null)return null;
			  OffsetPath[]offsets=new OffsetPath[paths.length];
			  for(int i=0;i<offsets.length;i++){
			  	TreePath path=paths[i];
		    	try{
						offsets[i]=new NodePath(path.getPath());
		    	}catch(IllegalStateException e){
		    		if(false)throw new RuntimeException("PathTree.storeExpansionPaths: ",e);
		    		else if(false)Util.println("PathTree.storeExpansionPaths: "+e);
		    		path=true?path:new TreePath(path.getParentPath());
		    	}
			  }
				return offsets;
			}
		};
	}
	void restoreExpansionPaths(){
		TypedNode root=(TypedNode)getModel().getRoot();
		String key=inGraphWrapper?lastKey:newExpansionsKey(root);
		OffsetPath[]offsets=newOffsetsProvider(null).getForValues(key);
	  if(offsets==null)return;
	  for(OffsetPath o:offsets)try{
	  	expandPath(new TreePath(o.members(root)));
	  }
	  catch(Exception e){
	  	if(false)Util.println("PathTree.restoreExpansionPaths: ",Debug.info(this));
	  	return;
	  }
	}
	protected String newExpansionsKey(TypedNode root){
		return inGraphWrapper?lastKey:root.type()+"|"+root.title();
	}
	void expandOrCollapse(TreePath path,int level,boolean expand){
		Object last=path.getLastPathComponent();
		if(!(last instanceof TypedNode))return;
		expand&=level-->0;
		TypedNode node=(TypedNode)last;
		for(TypedNode child:node.children())
			expandOrCollapse(path.pathByAddingChild(child),level,expand);
		if(expand)expandPath(path);
		else collapsePath(path);
	}
	public Insets getInsets(){
		return new Insets(5,5,5,5);
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
