package facets.util;
import static facets.util.Util.*;
import facets.util.app.AppValues;
import java.io.ByteArrayOutputStream;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.StringTokenizer;
/**
Utilities for use during development. 
 */
public final class Debug{
	public static final int HASH_MASK=0xfff;
	public static boolean memCheck=false,mbAsKb=false;
	/** Global flag.*/
	public static boolean natureDebug=true;
	public static Thread[]getSortedThreads(){
		ArrayList<Thread> threads=new ArrayList(Thread.getAllStackTraces().keySet());
		Collections.sort(threads,new Comparator(){
			@Override
			public int compare(Object o,Object p){
				return o.toString().compareTo(p.toString());
			}
		});
		return threads.toArray(new Thread[]{});
	}
	public static String mb(long bytes){
		return Doubles.sf(bytes/1024/(mbAsKb?1d:1024d))+(mbAsKb?"K":"");
	}
	public static String exceptionInfo(Exception e,String title){
		if(e==null)throw new IllegalArgumentException("Null e for title="+title);
		StringBuilder info=new StringBuilder(title+"\n");
		info.append(e+"\n");
		for(StackTraceElement line:Arrays.copyOf(e.getStackTrace(),5))
			info.append(line+"\n");
		info.append(e.getCause()+"\n");
		return info.toString();
	}
	/**
	A value that uniquely identifies the object within its class. 
	@return where available {@link Identified#identity()},
	otherwise the truncated {@link Object#hashCode()}.  
	 */
	public static String id(Object object){
		return true&&object instanceof Identified?("#"+((Identified)object).identity())
		:(""+(object.hashCode()&HASH_MASK));
	}
	/**
	Returns basic information about an object's type and identity. 
	<p>This will be some combination of
<ul>
	<li>the non-trivial simple class name
	<li>{@link #id(Object)}
	<li>{@link Titled#title()} if available
	</ul>
		 */
	public static String info(Object o){
		if(o==null)return "null";
		if(false&&o instanceof String)return"String length="+((String)o).length()+id(o);
		return (false?o.getClass().getSimpleName():helpfulClassName(o))+" "+
			id(o)+(o instanceof Titled?" "+((Titled)o).title():false?" "+o:"");
	}
	/**
	Returns an array of <code>info</code>s. 
	 */
	public static String arrayInfo(Object[]array){
		if(array==null)return "null";
		StringBuilder sb=new StringBuilder();
		for(int i=0;i<array.length;i++)
			sb.append((array[i]!=null?info(array[i]):"null")
					+(i<array.length-1?"\n":""));
		return sb+"";
	}
	public static String[]stackTraceLines(Throwable t){
		StackTraceElement[]stackTrace=t.getStackTrace();
		String[]lines=new String[stackTrace.length];
		for(int i=0;i<lines.length;i++)
			lines[i]="at "+stackTrace[i];
		return lines;
	}
	/**
	Prints a useful,truncated stack trace. 
	@param printLines how many lines of the trace to print
	 */
	public static void printStackTrace(int printLines){
		Util.println(parseStackTrace(printLines,"xxxxxxxxxxxxxx",false));
	}
	/**
	Prints useful stack trace up to eg a method name. 
	@param stopText occurring in a line stops further lines appearing
	 */
	public static void printStackTrace(String stopText){
		Util.println(parseStackTrace(1000,stopText,false));
	}
	public static void printThisTrace(Object src,int printLines){
		Util.println(info(src)+":\n"+parseStackTrace(printLines,"xxxxxxxxxxxxxx",false));
	}
	public static void printThisTrace(Object src,String stopText){
		Util.println(info(src)+":\n"+parseStackTrace(0,stopText,false));
	}
	public static String[]readTraceLines(Throwable t,int stopLines,String stopText,
			int omitLines){
		ByteArrayOutputStream out=new ByteArrayOutputStream();
		t.printStackTrace(new PrintStream(out));
		StringTokenizer parser=new StringTokenizer(out.toString(),"\n\r\t");
		int limit=stopLines+omitLines;
		ItemList<String>lines=new ItemList(String.class);
		for(int i=0;i<limit&&parser.hasMoreTokens();i++){
			String line=parser.nextToken();
			if(i<omitLines) continue;
			else lines.addItem(line);
			if(line.indexOf(stopText)>0)break;
		}
		return lines.items();
	}
	public static String parseStackTrace(int stopLines,String stopText,boolean noLinks){
		String[]lines=readTraceLines(new Throwable(),stopLines,stopText,3);
		StringBuffer output=new StringBuffer("\n");
		for(int i=0;i<lines.length&&lines[i]!=null;i++){
			String append=noLinks?lines[i].replaceAll("at([^\\(]*)\\([^\\)]*\\)","$1")
					:lines[i].replaceAll("^[^A-Z]+([A-Z])","$1");
			output.append(append);
			if(i<lines.length-1)output.append("\n");
		}
		if(false&&stopLines>1)output.append("\n");
		return output.toString();
	}
	public static void trapCall(){
		if(true)printStackTrace(5);
		else throw new RuntimeException("Load");
	}
	public static void memCheck(String header,boolean withGc){
		if(!memCheck)return;
		Runtime rt=Runtime.getRuntime();
		long usedBefore=rt.totalMemory()-rt.freeMemory();
		if(withGc)rt.gc();
		long usedAfter=rt.totalMemory()-rt.freeMemory();
		String msg=header+(withGc?("before gc "+mb(usedBefore)+
				", after "+mb(usedAfter)):(" "+mb(usedAfter)));
		Util.println(msg);
		
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

