package facets.util;
import static facets.util.Util.*;
import facets.util.datatree.DataNode;
import facets.util.datatree.Nodes;
import facets.util.datatree.TypedNode;
import java.util.ArrayList;
import java.util.List;
/**
Encapsulates assembly of HTML code. 
 */
public abstract class HtmlBuilder extends Tracer{
	@Override
	protected void traceOutput(String msg){
		if(false)super.traceOutput(msg);
	}
	final public String buildPage(){
		String top=newPageTop(),tail=newPageTail(),content=buildPageContent();
		trace(".buildPage: content lines="+content.split("\n").length);
		return top+content+tail;
	}
	public String newPageTop(){
		String title=getTitle(),styles=getPageStyles(),
			width=getPageTableWidth(),color=getPageColor();
		return"<html><head><title>"+title+"</title>\n" +styles+"</head>\n"+
	"<body leftmargin=0 rightmargin=0 topmargin=0 bottommargin=0 " +
	"bgcolor=\""+color +"\">\n"+
	"<table width=" +width+" border=0><tr><td>\n" +
	"<!--end of HtmlBuilder.newPageTop()-->\n";
	}
	protected String getTitle(){
		return "getTitle()";
	}
	public String getPageStyles(){
		ItemList<String>out=new ItemList(String.class);
		for(String raw:getStyles()){
			String[]splits=raw.split("\\{");
			for(String selector:splits[0].split(","))
				out.add(selector+"{"+splits[1]);
		}
		trace(".getPageStyles: out=",out);
		return "<style type=\"text/css\">\n"+
				Strings.linesString(out.items())+"\n</style>\n";
	}
	protected String[]getStyles(){
		double points=getPoints(),para=getBeforePara();
		return new String[]{
	"h1{font-family:sans-serif;font-size:" +sf(points*1.33)+"pt;\n" +
	"  margin-top:" +para+"pt;margin-left:0pt;margin-bottom:0pt;}",
	"dl{font-family:sans-serif;font-size:" +sf(points*1.1)+"pt;\n" +
	"  margin-top:" +para*2+"pt;margin-left:0pt;margin-bottom:0pt;}",
	"p,dd{font-family:sans-serif;font-size:" +sf(points)+"pt;line-height:normal;\n" +
	"  margin-top:" +para+"pt;margin-left:0pt;margin-bottom:0pt;}",
	"li{font-family:sans-serif;font-size:" +sf(points)+"pt;\n"+
	"  margin-top:" +para+"pt;margin-left:0pt;margin-bottom:0pt;"+
	"  list-contentStyle-position:outside;}",
	"ul{font-family:sans-serif;font-size:+" +sf(points)+"pt;\n"+
	"  margin-top:" +para+"pt;margin-left:24pt;margin-bottom:0pt;}"
		};
	}
	protected double getPoints(){
		return 12;
	}
	protected double getBeforePara(){
		return getPoints()/2;
	}
	protected String getPageColor(){
		return"#ffffff";
	}
	protected String getPageTableWidth(){
		return"100%";
	}
	protected String buildPageContent(){
		return"<p>buildPageContent()";
	}
	public String newPageTail(){
		return"\n<!--start of HtmlBuilder.newPageTail()-->\n" +
				"</td></tr>\n</table>\n</body>\n</html>";
	}
	public static String tagContents(final String text,String tagName){
		return text.replaceAll("(?s).+<"+tagName+">(.*)</"+tagName+">.*","$1").trim();
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
