package facets.util;
import static facets.util.NumberValues.ValueType.*;
import static facets.util.Strings.*;
import static java.lang.Math.*;
import static java.lang.System.*;
import static java.util.Arrays.*;
import facets.util.datatree.ValueNode;
import java.util.Arrays;
/**
Encapsulates management of numeric values stored in a {@link ValueNode}. 
 */
public abstract class NumberValues extends Tracer{
	enum ValueType{Int,Ints,Doubles}
	private final ValueType type;
	private final Object defaultValue;
	private final String keyTop;
	private final int checkCount;
	private Object value;
	private boolean hasSessionValue;
	protected NumberValues(Object defaultProposal,int checkCount,String keyTop){
		this.keyTop=keyTop;
		this.checkCount=checkCount;
		type=defaultProposal instanceof Integer?Int
				:defaultProposal instanceof int[]?Ints
				:defaultProposal instanceof double[]?Doubles
				:null;
		if(type==null)
			throw new RuntimeException("Not implemented for " +defaultProposal.getClass());
		defaultValue=type==Int?defaultProposal
				:type==Ints?validDefaultInts((int[])defaultProposal)
				:validDefaultDoubles((double[])defaultProposal);
	}
	protected abstract ValueNode store();
	protected abstract String keyTailSession();
	protected abstract String keyTailDefaults();
	@Override
	protected void traceOutput(String msg){
		if(false)super.traceOutput(msg);
	}
	protected double[]validDefaultDoubles(double[]proposed){
		if(proposed.length>=checkCount)return proposed;
		double[]defaults=new double[checkCount];
		arraycopy(proposed,0,defaults,0,min(proposed.length,checkCount));
		trace(": doubles count="+checkCount+
				" needed="+checkCount+" add=",doublesString(defaults));
		return defaults;
	}
	protected int[]validDefaultInts(int[]proposed){
		if(proposed.length>=checkCount)return proposed;
		int[]defaults=new int[checkCount];
		arraycopy(proposed,0,defaults,0,min(proposed.length,checkCount));
		trace(": int count="+checkCount+
				" needed="+checkCount+" add=",intsString(defaults));
		return defaults;
	}
	public boolean hasSessionValue(){
		return hasSessionValue;
	}
	public Object getValue(boolean getDefault){
		try{
			return(!getDefault&&(hasSessionValue=(getCount(keyTailSession())>=checkCount)))
				||getCount(keyTailDefaults())>=checkCount?value:defaultValue;
		}catch(Exception e){
			return defaultValue;
		}
	}
	public void putValue(Object value,boolean setDefault){
		ValueNode store=store();
		store.put(keyTop+keyTailSession(),value);
		if(setDefault)store.put(keyTop+keyTailDefaults(),value);
		hasSessionValue=true;
	}
	private int getCount(String keyTail){
		String key=keyTop+keyTail;
		return type==Int?intCount(key):type==Ints?intsCount(key)
			:doublesCount(key);
	}
	private int intCount(String key){
		int check=store().getInt(key);
		value=check;
		return check==ValueNode.NO_INT?0:1;
	}
	private int intsCount(String key){
		int[]check=store().getInts(key);
		value=copyOfRange(check,0,checkCount);
		return check.length;
	}
	private int doublesCount(String key){
		double[]check=store().getDoubles(key);
		value=copyOfRange(check,0,checkCount);
		return check.length;
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

