package facets.util;
import java.util.Collection;
/**
Utility superclass that can issue trace messages.
 */
public abstract class Tracer{
	final public void trace(String msg){
		traceOutput(msg);
	}
	final public void trace(String msg,Throwable t,boolean stack){
		if(stack&=t!=null){
			traceOutput(msg);		
			t.printStackTrace();
		}
		else traceOutput(msg+traceObjectText(t));		
	}
	final public void trace(String msg,Object o){
		traceOutput(msg+traceObjectText(o));		
	}
	final public void trace(String msg,Collection c){
		traceOutput(msg+traceArrayText(c.toArray()));		
	}
	final public void trace(String msg,Object[]array){
		traceOutput(msg+traceArrayText(array));		
	}
	final public void traceDebug(String msg,Object o){
		traceOutput(msg+Debug.info(o));		
	}
	final public void traceDebug(String msg,Object[]array){
		traceOutput(msg+(false?Debug.info(array):Debug.arrayInfo(array)));		
	}
	/**
	Outputs complete trace messages to console or elsewhere. 
	<p>Default prepends helpful classname to message.  
	@param msg passed from one of the <code>public</code> methods
	 */
	protected void traceOutput(String msg){
		traceOutputWithClass(msg);
	}
	final protected String traceObjectText(Object o){
		return o==null?null:o.toString();
	}
	private String traceArrayText(Object[]array){
		return Util.arrayPrintString(array);
	}
	final public void traceOutputWithClass(String msg){
		System.out.println((false?getClass().getSimpleName()
				:Util.helpfulClassName(this))+msg);
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
