package facets.util.app;
import static facets.util.app.Events.*;
import facets.util.Debug;
import facets.util.Times;
import facets.util.Tracer;
import java.util.Timer;
import java.util.TimerTask;
/**
Sets the application busy cursor. 
<p>A {@link BusyCursor} passed to {@link AppWatcher#pushCursor(BusyCursor)}
will call {@link #setCursor(boolean)} whenever the {@link AppWatcher} senses significant
change in the application's busy state.
<p>Concrete instances must implement {@link #setCursor(boolean)} for a particular
combination of surface (eg application or dialog) and GUI toolkit.  
 */
public abstract class BusyCursor extends Tracer{
	private final static int onWait=100,offWait=250;
	private final static Timer waits=new Timer("AppCursor.waits");
	private static int calls;
	private boolean state,busyThen;
	private TimerTask waiter;
	final protected void traceOutput(String msg){
		if(true)return;
		msg=BusyCursor.class.getSimpleName()+msg;
		if(true)System.out.println(msg);
		else Times.printElapsed(msg);
	}
	final void checkBusyState(final boolean busy){
		if(busyThen==busy)return;
		busyThen=busy;
		int call=calls++;
		trace(newCursorMsg(call,"checkBusyState",busy));
		if(state==busy&&waiter==null)return;
		else if(state!=busy&&waiter==null)waits.schedule(waiter=new TimerTask(){
			int call=calls++;
			public void run(){
				if(false&&this!=waiter)throw new IllegalStateException(
						"Different waiters in "+Debug.info(this));
				trace(".waiter.run: ",this);
				switchCursor(busy);
			}
			public boolean cancel(){
				trace(".waiter.cancel: ",this);
				boolean cancel=super.cancel();
				waiter=null;
				return cancel;
			}
			public String toString(){
				return busy+" #"+call;
			}
		},busy?onWait:offWait);
		else if(state==busy&&waiter!=null)waiter.cancel();	
		trace(newCursorMsg(call,"~checkBusyState",busy));
	}
	public final void forceSwitch(boolean busy){
		trace(newCursorMsg(calls++,"forceCursor",busy));
		if(busy!=state)switchCursor(busy);
		busyThen=true;
	}
	private void switchCursor(boolean busy){
		int call=calls++;
		trace(newCursorMsg(call,"SWITCH_CURSOR ",busy));
		if(waiter!=null)waiter.cancel();
		setCursor(state=busy);
		if(trace&&events&&busy)traceEvent("Cursor switched busy="+state);
		else trace(newCursorMsg(call,"~switchCursor ",busy));
	}
	/**
	Set the busy cursor over the required part of the application surface. 
	 */
	protected abstract void setCursor(boolean busy);
	private String newCursorMsg(int call,String in,boolean busy){
		return " #"+call+" "+in+": busy="+busy+" state="+state+" waiter="+waiter;
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
