package facets.util.app;
import static facets.util.Debug.*;
import static facets.util.Util.*;
import static facets.util.app.AppValues.*;
import facets.util.Debug;
import facets.util.Times;
import facets.util.Tracer;
import facets.util.datatree.ValueNode;
import java.util.Timer;
import java.util.TimerTask;
/**
Checks memory availability, especially for use by 
{@link ProvidingCache}.
 */
public final class MemoryChecks extends Tracer{
	public static final int maxMemMbDefault=200;
	public static final String NATURE_MEM_MB="memMaxMb",ARG_TRACE="traceMem";
	private final static Runtime rt=Runtime.getRuntime();
	private static final int setMaxPct=10,safeFreePct=20;
	private long maxMem,checkedFree,total,free,freeThen=-1*MB,freePct=100;
	private static Timer checker=new Timer(MemoryChecks.class.getSimpleName());
	private static TimerTask check;
	private final boolean timing=false&&Times.times;
	private final AppValues values;
	private final boolean traceMem;
	@Override
	protected void traceOutput(String msg){
		if(!traceMem)return;
		String text=MemoryChecks.class.getSimpleName()+": "+msg;
		if(timing)Times.printElapsed(text);
		else System.out.println(text);
	}
	public MemoryChecks(AppValues values){
		if((this.values=values)==null)throw new IllegalArgumentException(
				"Null values in "+Debug.info(this));
		ValueNode nature=values.nature(true);
		maxMem=nature.getOrPutInt(NATURE_MEM_MB,maxMemMbDefault)*MB;
		traceMem=nature.getBoolean(ARG_TRACE);
	}
	public ProvidingCache newMemoryCache(AppWatcher watcher){
		return new ProvidingCache(-1,watcher){
			@Override
			protected void traceOutput(String msg){
				MemoryChecks.this.trace(msg);
			}
			@Override
			protected void processingRequest(boolean starting){
				if(check!=null)check.cancel();
				if(!starting)startChecks();
			}
			@Override
			protected boolean checkItemBytes(){
				return true;
			}
			@Override
			protected long getBytesAvailable(long itemMems){
				return MemoryChecks.this.checkFree(" for itemMems="+mb(itemMems));
			}
		};
	}
	public void startChecks(){
		int period=3;
		checker.schedule(check=new TimerTask(){
			@Override
			public void run(){
				updateStatus();
				if(timing)trace(newStatusText(true));
				rt.gc();
				updateStatus();
				if(false&&maxMem<total||freePct<setMaxPct)setAndStoreMaxSafe();
				updateStatus();
				if(timing||freeThen/MB!=free/MB)trace(newStatusText(true));
				freeThen=free;
			}
		},0*1000,period*1000);
	}
	private long checkFree(String debugMsg){
		updateStatus();
		if(checkedFree<10){
			trace("rechecking for "+newStatusText(true));
			rt.gc();
			updateStatus();
		}
		trace(newStatusText(true)+debugMsg);
		return checkedFree;
	}
	private void updateStatus(){
		total=rt.totalMemory();
		free=rt.freeMemory();
		freePct=intPct(free,total);
		checkedFree=freePct<safeFreePct?0:(maxMem-total+free);
	}
	public String newStatusText(boolean full){
		return(full?("maxMem="+mb(maxMem)+" checkedFree="+mb(checkedFree)):"")+
		(" total="+mb(total)+" free="+mb(free)+" free%="+freePct+" ");
	}
	public static void checkStatus(String msg){
		long total=rt.totalMemory(),free=rt.freeMemory(),freePct=intPct(free,total);
		System.out.println(msg+
				":\n\tMemory MB total="+mb(total)+" free="+mb(free)+" free%="+freePct);
	}
	private void doStartupProbe(){
		boolean slow=maxMem==0;
		setAndStoreMaxSafe();
		while(freeThen/MB!=free/MB&&freePct>setMaxPct){
			freeThen=free;
			updateStatus();
			trace("Probing: "+newStatusText(true));
			if(maxMem<total)maxMem=total;
			updateStatus();
			int probe=(int)(checkedFree/100*98/MB);
			try{
				byte[][]allocate=new byte[probe][MB];
			} 
			catch(OutOfMemoryError e){
				System.err.println(e+": probe="+probe);
				rt.exit(-1);
			}
			if(slow)rt.gc();
			updateStatus();
			trace("Probed: probe="+probe+" "+newStatusText(true));
			slow|=freePct>setMaxPct;
		}
		setAndStoreMaxSafe();
	}
	private void setAndStoreMaxSafe(){
		values.state(PATH_CORE_APP).put(NATURE_MEM_MB,(int)((maxMem=total)/MB));
		values.tryWrite(true?""
			:("setMaxSafeMb="+values.state(PATH_CORE_APP).getInt(NATURE_MEM_MB)+": "));
	}
	static void main(String[]args){
		final AppValues values=new AppValues(MemoryChecks.class){
			protected void traceOutput(String msg){
				if(false)traceOutputWithClass(": "+msg);
			}
		};
		values.readValues(args);
		new MemoryChecks(values).doStartupProbe();
		System.exit(0);
	}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
