package facets.util.geom;
import static facets.util.geom.Angle.*;
import facets.util.NumberPolicy;
import facets.util.NumberPolicy.Ticked;
/**
Validation and display of angle values.  
<p>{@link AnglePolicy} supplies policy for
angle values on a tick-based scale that can offer either
full-range or local adjustment. 
 */
public class AnglePolicy extends Ticked{
  /**
	Unique public constructor. 
	<p>Constructs a full-range policy displaying -180 to +180 degrees. 	
	 */
	public AnglePolicy(){this(Double.NaN);}
  private AnglePolicy(double range) {
		super(-CIRCLE_DEGREES / 2, CIRCLE_DEGREES / 2, range);
	}
  /**
	Re-implementation to return <code>true</code>. 
	 */
	final public boolean canCycle(){return true;}
  final public int format(){return FORMAT_DECIMALS_0;}
	public String[] incrementTitles() {
		return new String[]{"Anticlockwise","Clockwise"};
	}
  public int labelSpacing(){return 6;}
  /**
  Returns a local adjustment policy with a 60� range and its snap type
  coupled to that of its parent. 
  @see facets.util.NumberPolicy.Ticked#labelSpacing()
      */
  public Ticked localTicks(){
    final Ticked parent=this;
    return new AnglePolicy(CIRCLE_DEGREES/4){
      public int labelSpacing(){return 3;}
    	public int snapType(){return parent.snapType();}
    	public int tickSpacing(){return 5;}
    };
  }
	public int snapType(){return SNAP_TICKS;}
	public int tickSpacing(){return 15;}
  final protected boolean reverseIncrements(){return false;}
}/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
