package facets.util.geom;
import facets.util.Doubles;
/**
Abstract 2D point. 
 */
final public class Point implements java.io.Serializable{
  private Vector at;
  public int hashCode(){
  	return at.hashCode();
  }
  public Point(){at=new Vector();}
  public Point(double x,double y){set(x,y);}
	public Point(double[]vals){
		this(vals[0],vals[1]);
		if(vals.length>2)throw new IllegalArgumentException(
				"Too many values length="+vals.length);
	}
  public Point(Point src){this(src.at.x,src.at.y);}
  public Point(Vector src){this(src.x,src.y);}
  public Vector at(){return new Vector(at);}
  public double distance(Point from){return at.minus(from.at).reach();}
  final public boolean eq(double a,double b){return Math.abs(a-b)<0.01;}
	public boolean equals(Object o){
		return o==null?false:at.equals(((Point)o).at);
	}
  public int intX(){return(int)at.x;}
  public int intY(){return(int)at.y;}
  public Vector jumpFrom(Point point){return at.minus(point.at);}
  public Point newPoint(){return new Point(this);}
  public void rotate(Point about,Angle angle){
    double cos=at.x-about.at.x,sin=at.y-about.at.y,r=Doubles.hypotenuse(cos,sin);
    if(angle.withinTolerance(Angle.NIL)||r<.00001)return;
    double addTan=r*angle.tan(),newR=Doubles.hypotenuse(addTan,r),
    newSin=sin+cos*addTan/r,newCos=cos-sin*addTan/r;
    newCos*=r/newR;newSin*=r/newR;
    if(angle.cos()<0){newCos*=-1;newSin*=-1;}
    set(about.at.x+newCos,about.at.y+newSin);
  }
  public void set(double x,double y){at=new Vector(x,y);}
  public void set(Vector to){set(to.x,to.y);}
  public void shift(Vector shift){set(at.plus(shift));}
  public Point shifted(Vector shift){return new Point(at.plus(shift));}
	public double storeX(){
		return Doubles.fx(at.x);
	}
	public double storeY(){
		return Doubles.fx(at.y);
	}
  public String toString(){return at.toString();}
  public double x(){return at.x;}
  public double y(){return at.y;}
  Angle bearing(Point from){return at.minus(from.at).bearing();}
	public double[]storeXY(){
		return new double[]{storeX(),storeY()};
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

