package facets.util.treecode;
import static facets.util.datatree.TypedNode.*;
import facets.util.Debug;
import facets.util.Objects;
import facets.util.Regex;
import facets.util.Titled;
import facets.util.Tracer;
import facets.util.datatree.DataNode;
import facets.util.datatree.Nodes;
import facets.util.datatree.TypedNode;
import java.util.Comparator;
/**
Wraps a {@link DataNode} containing its code and has a (possibly empty) {@link Label}. 
 */
public abstract class TreeCoded extends Tracer implements Titled,Comparable<TreeCoded>{
	private static final String PREFIX_DEBUG="*";
	/**
	Identifies a {@link TreeCoded}. 
	<p>As well as carrying general-purpose text, a {@link Label} can identify either 
	the {@link TreeCoded} that returns it, or
	another to be referenced within a {@link TreeCodeContext}. 
	</ul>
	 */
	public static final class Label{
		public static final Label NONE=new Label(TypedNode.UNTITLED);
		public final String text;
		Label(String text){
			this.text=text;
		}
		@Override
		public int hashCode(){
			return text.hashCode();
		}
		@Override
		public boolean equals(Object that){
			return text.equals(((Label)that).text);
		}
		@Override
		public String toString(){
			return Debug.info(this)+" text="+text;
		}
	}
	public final DataNode source;
	public final Label label;
	public final boolean debug;
	public final TreeCodeType treeType;
	private final String ancestry;
	@Override
	protected void traceOutput(String msg){
		super.traceOutput(" ["+ancestry+"]"+msg);
	}
	protected TreeCoded(DataNode source,TreeCodeType type){
		this.source=source;
		this.treeType=type;
		String title=source.title();
		debug=title.startsWith(PREFIX_DEBUG);
		label=new Label(title.replaceAll("^\\Q"+PREFIX_DEBUG+"\\E",""));
		ancestry=Regex.replaceAll(Objects.toString(Nodes.ancestry(source),">\n\t"),
				"No values","","#\\d+","",
				"(?s)^.*Form\\s*>\\s*","",
				"","").trim();
	}
	@Override
	final public String title(){
		return label.text;
	}
	@Override
	public String toString(){
		return source.type()+" '"+title().replace(UNTITLED,"")+"'";
	}
	@Override
	public int compareTo(TreeCoded q){
		int byType=treeType.name.compareTo(q.treeType.name);
		return byType!=0?byType:label.text.compareTo(q.label.text);
	}
}
/*/*
This file forms part of Version 0.8.79 of Facets, a framework demonstrating 
the Superficial approach to GUI-driven interactive applications.  
http://superficial.sourceforge.net 				
Copyright (C) 2011  David M Wright 
This library is free software; you can redistribute it and/or modify it under 
the terms of the GNU Lesser General Public License as published by the 
Free Software Foundation; either release 3 of the License, or (at your 
option) any later release.
This library is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR 
A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more 
details.
You should have received a copy of the GNU Lesser General Public License along 
with this library; if not, write to the Free Software Foundation, Inc., 
59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

